/*
 * Copyright (C) 2005, Laszlo Pere <pipas@linux.pte.hu>
 * $Id: background.h,v 1.1 2006/02/24 17:35:38 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: background.h,v $
 * Revision 1.1  2006/02/24 17:35:38  pipas
 * Initial revision
 *
 */

#ifndef BACKGROUND_H
#define BACKGROUND_H

#include <gnome.h>

/**
 * BackgroundMethod:
 * @MethodGraphics: using the CUI as a graphical user interface
 * @MethodSelection: using the CUI to select items
 * @MethodEditor: using the CUI as a text editor
 *
 */
typedef enum {
	MethodGraphics,
	MethodSelection,
	MethodEditor
} BackgroundMethod;

typedef enum {
	CommandNone,
	CommandNewFile,
	CommandNewDirectory
} StoredCommand;

/**
 * Background:
 * @bg_widgets: a list of CuiWidgets
 * @colormap: the colormap to draw objects (FIXME: do we need this?)
 * @widget: the terminal widget
 * @text_view: same as the widget but as a #GtkTextView
 * @text_buffer: the terminal's text_buffer (FIXME: do we need this?)
 * @window: the window to draw in (A #GtkTextView has 3 windows!)
 * @gc: the graphics context to draw
 * @mouse_cursor_arrow: an arrow like mouse cursor
 * @mouse_cursor_hand: a hand mouse cursor to use
 * @mouse_cursor_text: a mouse cursor like the text editors used to show
 * @method: the user communication method of the background
 */
typedef struct _Background Background;

struct _Background {
	/*<private>*/
	gint              double_click_time;
	gint              last_mouse_x;
	gint              last_mouse_y;
	gint              last_button;
	guint32           last_click_time;	
	gint              click_count;
	gboolean          still_holding;
	GdkAtom           match;
	gpointer          dnd_target_widget;
	gpointer          dnd_source_widget;
	gpointer          stored_widget;
	gboolean          view_active_areas;
	gboolean          view_sizebars;
	StoredCommand     stored_command;
	/*<public>*/
	GList            *bg_widgets;
	GdkColormap      *colormap;
	
	GtkTextView      *text_view, *secondary_text_view;
	GtkWidget        *widget;
	GdkWindow        *window, *secondary_window;
	
	GtkTextBuffer    *text_buffer;
	GdkGC            *gc;
	GdkCursor        *mouse_cursor_arrow;
	GdkCursor        *mouse_cursor_hand;
	GdkCursor        *mouse_cursor_text;
	BackgroundMethod  method;
};

/**
 * BackgroundViewOptions:
 * @ActiveAreas: all active areas
 * @Sizebars: the sizebars (displayed for ls -l for example)
 */

typedef enum {
	ActiveAreas,
	Sizebars
} BackgroundViewOption;

/****************************************************************************
 * Functions handling whole background areas filled with background         *
 * elements.                                                                *
 *                                                                          *
 ***************************************************************************/
Background *background_new(GtkTextView *widget);
void background_set_secondary_text_view(Background *bg, GtkTextView *text_view);
void background_set_method(Background *bg, BackgroundMethod method);
void background_toggle_view_flag(Background *bg, BackgroundViewOption option);
void background_store_widget(Background *bg, gpointer widget);
void background_store_command(Background *bg, StoredCommand command);
gint cui_background_append_widget(Background *bg, gpointer widget);
gpointer *background_get_last_dnd_target(Background *background);
gpointer *background_get_last_dnd_source(Background *background);
gpointer *background_get_stored_widget(Background *background);
gboolean background_get_view_flag(Background *bg, BackgroundViewOption option);
gboolean background_check_stored_command(Background *bg, StoredCommand command);


gboolean
background_button_press_event(
		Background     *bg,
		GtkWidget      *widget, 
		GdkEventButton *event, 
		gpointer        user_data);
gboolean
background_button_release_event(
		Background      *bg,
		GtkWidget       *widget, 
		GdkEventButton  *event, 
		gpointer         user_data);
gboolean
background_motion_notify_event(
		Background     *bg,
		GtkWidget      *widget, 
		GdkEventMotion *event, 
		gpointer        user_data);
gboolean
background_leave_notify_event(
		Background       *bg,
		GtkWidget        *widget, 
		GdkEventCrossing *event, 
		gpointer          user_data);

gboolean 
background_expose_event (
		GtkWidget       *widget, 
		GdkEventExpose  *event, 
		gpointer         user_data);

gboolean 
background_dnd_motion_notify(
		GtkWidget      *widget,
		Background     *background, 
		GdkDragContext *drag_context, 
		gint            x, 
		gint            y);
#endif
