/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: command_generator.h,v 1.1 2006/02/24 17:34:33 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: command_generator.h,v $
 * Revision 1.1  2006/02/24 17:34:33  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include "terminal.h"
#include "filename.h"
#include "background.h"

#ifndef COMMAND_GENERATOR_H
#define COMMAND_GENERATOR_H

/**
 * CuiCommandOpcode:
 * @FileOpen:
 * @FileRefresh:
 * @FileOpenForEdit:
 * @FileOpenForView:
 * @FileProcess:
 * @FileOpenWithVim:
 * @FileOpenWithEmacs:
 * @FileNewRegularFile:
 * @FileNewDirectory:
 * @FileNewTarArchive:
 * @FileNewTarGzArchive:
 * @FileNewTarBzArchive:
 * @FileDelete:
 * @FileMoveToTrash:
 * @FileUncompress:
 * @FileShowProperties:
 * @FileClipBoardCut:
 * @FileClipBoardCopy:
 * @FileClipBoardPaste:
 * @FileSetModeToFile:
 * @FileSetModeToDir:
 * @FileSetModeToProgram:
 * @FileViewACLList: View Access Control List entries.
 * @FileGrantReadUser:
 * @FileRevokeReadUser:
 * @FileGrantWriteUser:
 * @FileRevokeWriteUser:
 * @FileGrantExecuteUser:
 * @FileRevokeExecuteUser:
 * @FileGrantReadGroup:
 * @FileRevokeReadGroup:
 * @FileGrantWriteGroup:
 * @FileRevokeWriteGroup:
 * @FileGrantExecuteGroup:
 * @FileRevokeExecuteGroup:
 * @FileGrantReadOthers:
 * @FileRevokeReadOthers:
 * @FileGrantWriteOthers:
 * @FileRevokeWriteOthers:
 * @FileGrantExecuteOthers:
 * @FileRevokeExecuteOthers:
 * @FileRename: These command has a char argument. 
 * @FileCopy:
 * @FileMove:
 * @FileChangeOwner:
 * @FileChangeGroup:
 * @FileChangeAccessDate:
 * @FileChangeModifyDate:
 * @FileChangeChangeDate:
 * @WindowRefresh: executes the command of the window to refresh the window content
 *
 * A simple command code.
 *
 */
typedef enum {
	FileOpen,
	FileRefresh,
	FileOpenForEdit,
	FileOpenForView,
	FileProcess,
	FileOpenWithVim,
	FileOpenWithEmacs,
	FileNewRegularFile,
	FileNewDirectory,
	FileNewTarArchive,
	FileNewTarGzArchive,
	FileNewTarBzArchive,
	FileDelete,
	FileMoveToTrash,
	FileUncompress,
	FileShowProperties,
	FileClipBoardCut,
	FileClipBoardCopy,
	FileClipBoardPaste,
	FileSetModeToFile,
	FileSetModeToDir,
	FileSetModeToProgram,
	FileViewACLList,
	FileGrantReadUser,
	FileRevokeReadUser,
	FileGrantWriteUser,
	FileRevokeWriteUser,
	FileGrantExecuteUser,
	FileRevokeExecuteUser,
	FileGrantReadGroup,
	FileRevokeReadGroup,
	FileGrantWriteGroup,
	FileRevokeWriteGroup,
	FileGrantExecuteGroup,
	FileRevokeExecuteGroup,
	FileGrantReadOthers,
	FileRevokeReadOthers,
	FileGrantWriteOthers,
	FileRevokeWriteOthers,
	FileGrantExecuteOthers,
	FileRevokeExecuteOthers,
	/* These commands has a char argument accepted when exeuting. */
	FileRename,               
	FileCopy,
	FileMove,
	FileChangeOwner,
	FileChangeGroup,
	FileChangeAccessDate,
	FileChangeModifyDate,
	FileChangeChangeDate, 
	/* These commands has an implicit char argument. */
	FileChangeACL,
	FileOpenWithLine,
	WindowDirectCommand,
	/* Window commands */
	WindowRefresh
} CuiCommandOpcode;

/**
 * CuiCommandType:
 * @FileCommand:
 * @FileCommandWithArg:
 * @WindowCommand:
 * @WindowCommandWithArg:
 *
 * A type to arrange the command opcodes into groups.
 */
typedef enum {
	FileCommand,
	FileCommandWithArg,
	WindowCommand,
	WindowCommandWithArg
} CuiCommandType;

/**
 * CuiCommand:
 * @type: the type of the command used to decide how to perform
 * @opcode: the operation code (what to do)
 * @parg1: a pointer argument 
 * @parg2: an optional second argument
 *
 */
struct _CuiCommand {
	CuiCommandType     type;
	CuiCommandOpcode   opcode;
	gpointer           parg1;
	gpointer 	   parg2;
};

typedef struct _CuiCommand CuiCommand;
	

#define CUI_GENERATOR_TYPE (cui_generator_get_type())
#define CUI_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_GENERATOR_TYPE, CuiGenerator))
#define CUI_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_GENERATOR_TYPE, CuiGeneratorClass))
#define CUI_IS_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_GENERATOR_TYPE))
#define CUI_IS_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), \
			CUI_GENERATOR_TYPE))
#define CUI_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), \
			CUI_GENERATOR_TYPE, CuiGeneratorClass))

/**
 * CuiGenerator:
 * @terminal: we send the command for this #Terminal
 * @os: the name of the operating system
 *
 */
typedef struct _CuiGenerator CuiGenerator;
struct _CuiGenerator
{
	/*< private >*/
	GObject parent;
	/*< public >*/
	gpointer     terminal;
	gchar       *os;
};

typedef struct _CuiGeneratorClass CuiGeneratorClass;
struct _CuiGeneratorClass {
	/*< private >*/
	GObjectClass parent_class;
	/*< public >*/
} ;

GType cui_generator_get_type(void);

CuiGenerator *cui_generator_new(void);
void cui_generator_set_terminal(CuiGenerator *generator, 
		const gpointer *terminal);
void cui_generator_send_shell_command(CuiGenerator *generator, 
		const char *command);

void cui_generator_emit_file_command( CuiGenerator *generator,
		CuiCommandOpcode command,
		CuiFilename *file);
void cui_generator_emit_file_command_with_arg(
		CuiGenerator *generator,
		CuiCommandOpcode command,
		CuiFilename *file,
		const gchar *text);

gint cui_generator_emit_commands(CuiGenerator *command_generator, GList *list);

#endif
