/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

#include "cuiterm.h"
#include "configure.h"
#include "support.h"
#include "interface.h"
#include "cuilexelements.h"
#include "background.h"
#include "cuiwindow.h"
#include "complex_widgets.h"
#include "command_generator.h"


CuiWidget *
cui_widgets_new_for_filename(
		CuiWindow     *window,
		CuiLexElement *lex)
{
	Background *background;
	CuiWidget  *widget;
	GtkWidget  *appbar;
	gchar      *tooltip;
	
	g_assert(CUI_IS_WINDOW(window));
	g_assert(lex != NULL);
	/*
	 * Collecting information.
	 */
	background = cui_widget_get_background(CUI_WIDGET(window));
	appbar = lookup_widget(background->widget, "appbar1");
	/*
	 * Action area for the username.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, lex->begin, lex->end);
	cui_widget_add_tag_by_name(widget, "username");
	cui_widget_set_active_style(widget, "username_active");
	tooltip = g_strdup_printf(
			_("'%s' is a username. Double click to get information about this user."),
			lex->text);
	cui_widget_set_tooltip(widget, tooltip);
	/*
	 * FIXME: Here we should ad the finger command with some 
	 * cui_widget_append_file_command like command.
	 */
	
	/*
	 * Menü for the username.
	 */
   	widget = cui_user_menu_new(lex->text);
	cui_widget_set_background(widget, background);
	cui_widget_set_area_by_marks(widget, lex->begin, lex->end);
}


CuiWidget *
cui_widgets_new_for_directory(
		CuiWindow     *window,
		CuiLexElement *lex)
{
	Background  *background;
	CuiFilename *fn;
	CuiWidget   *widget;
	GtkWidget   *appbar;
	gchar       *tooltip;
	
	g_assert(CUI_IS_WINDOW(window));
	g_assert(lex != NULL);
	/*
	 * Collecting information.
	 */
	background = cui_widget_get_background(CUI_WIDGET(window));
	appbar = lookup_widget(background->widget, "appbar1");
	/*
	 * Creating a new filename.
	 */
	fn = cui_filename_new(lex->text, window->wd->str); 
	cui_filename_set_filetype_dir(fn);
	/*
	 * Action area for the directory.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, lex->begin, lex->end);
	cui_widget_add_tag_by_name(widget, "directory");
	cui_widget_set_active_style(widget, "directory_active");
	tooltip = g_strdup_printf(
			_("The '%s' is a directory. Double click to open it."),
			lex->text);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_append_file_command(widget, FileOpen, fn);
	/*
	 * Menü for the username.
	 */
   	widget = cui_file_menu_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_area_by_marks(widget, lex->begin, lex->end);
	cui_file_menu_set_filename(CUI_FILE_MENU(widget), fn);
}


CuiWidget *
cui_widgets_new_for_fullpath(
		CuiWindow     *window,
		CuiLexElement *filenamelex,
		CuiLexElement *directorylex)
{
	Background  *background;
	CuiFilename *fn;
	CuiWidget   *widget;
	GtkWidget   *appbar;
	gchar       *tooltip;
	gchar       *tag, *active_tag;
	
	g_assert(CUI_IS_WINDOW(window));
	g_assert(filenamelex != NULL && directorylex != NULL);
	/*
	 * Collecting information.
	 */
	background = cui_widget_get_background(CUI_WIDGET(window));
	appbar = lookup_widget(background->widget, "appbar1");
	/*
	 * Creating a new filename.
	 */
	fn = cui_filename_new(filenamelex->text, directorylex->text); 
	/*
	 * Action area for the directory.
	 */
	if (cui_filename_is_dir(fn)){
		tag = "directory";
		active_tag = "directory_active";
	} else if (cui_filename_is_user_execute(fn) ||
			cui_filename_is_group_execute(fn) ||
			cui_filename_is_others_execute(fn) ){
		tag = "executable";
		active_tag = "executable_active";
	} else if (cui_filename_is_compressed(fn)){
		tag = "archive";
		active_tag = "archive_active";
	} else if (cui_filename_is_block_device(fn)) {
		tag = "block_device";
		active_tag = "block_device_active";
	} else if (cui_filename_is_character_device(fn)) {
		tag = "character_device";
		active_tag = "character_device_active";
	} else { 
		tag = "file";
		active_tag = "file_active";
	}
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, filenamelex->begin, filenamelex->end);
	cui_widget_add_tag_by_name(widget, tag);
	cui_widget_set_active_style(widget, active_tag);
	tooltip = g_strdup_printf(
			_("The '%s' is a directory entry. Double click to open it."),
			filenamelex->text);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_append_file_command(widget, FileOpen, fn);
	/*
	 * Menü for the username.
	 */
   	widget = cui_file_menu_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_area_by_marks(widget, filenamelex->begin, filenamelex->end);
	cui_file_menu_set_filename(CUI_FILE_MENU(widget), fn);
}

