/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: cuiwindow.c,v 1.1 2006/02/24 17:21:24 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: cuiwindow.c,v $
 * Revision 1.1  2006/02/24 17:21:24  pipas
 * Initial revision
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include "cuiterm.h"
#include "support.h"
#include "interface.h"
#include "cuiwindow.h"
#include "terminal.h"
#include "widgets.h"
#include "bash_parser_c.h"

#if DEBUG_ALL
#  define DEBUG
#endif

CuiWindow *window_for_lex = NULL;

int cat_parse(CuiWindow *window);
int df_parse(CuiWindow *window);
int exit_parse(CuiWindow *window);
int simple_parse(CuiWindow *window);
int fdisk_parse(CuiWindow *window);
int finger_parse(CuiWindow *window);
int general_parse(CuiWindow *window);
int getfacl_parse(CuiWindow *window);
int ls_parse(CuiWindow *window);
int make_parse(CuiWindow *window);
int man_parse(CuiWindow *window);
int ps_parse(CuiWindow *window);
int stat_parse(CuiWindow *window);
int tree_parse(CuiWindow *window);
int who_parse(CuiWindow *window);
int w_parse(CuiWindow *window);
int du_parse(CuiWindow *window);

//static void ls_parse_output(CuiWindow *window);

typedef struct executor {
	char *name;
	int (*executor)(CuiWindow *window);
	gchar *window_title;
	gchar *color, *lcolor, *hcolor;
} executor;

/*
 * FIXME: This should be a GList or something so we could create new elements 
 * and i18n character strings in runtime.
 */
executor executors[] = {
	{ "ls",      ls_parse,      "Directory Content",   "#6281ff", "#5975f7", "#8ba2ff"},
	{ "getfacl", getfacl_parse, "Access Control List", "#6281ff", "#5975f7", "#8ba2ff"},
	{ "tree",    tree_parse,    "File Tree View",      "#6281ff", "#5975f7", "#8ba2ff"},
	{ "df",      df_parse,      "Disk Usage",          "#6281ff", "#5975f7", "#8ba2ff"},
	{ "fdisk",   fdisk_parse,   "Partition Table",     "#6281ff", "#5975f7", "#8ba2ff"},
	{ "stat",    stat_parse,    "File State",          "#6281ff", "#5975f7", "#8ba2ff"},
	{ "cat",     cat_parse,     "File Content",        "#f4f244", "#f4f22f", "#f4f26d"},
	{ "make",    make_parse,    "Projekt Management",  "#f4f244", "#f4f22f", "#f4f26d"},
	{ "ps",      ps_parse,      "Process Status",      "#53ff67", "#20ff3a", "#99ffa5"},
	{ "who",     who_parse,     "Loggen on Users",     "#f371f1", "#f357f1", "#f3a0f2"},
	{ "w",       w_parse,       "Loggen on Users",     "#f371f1", "#f357f1", "#f3a0f2"},
	{ "finger",  finger_parse,  "User Properties",     "#f371f1", "#f357f1", "#f3a0f2"},
	{ "exit",    exit_parse,    "Exit Shell",          "#f371f1", "#f357f1", "#f3a0f2"},
	{ "whoami",  simple_parse,  "Effective UserID",    "#6281ff", "#5975f7", "#8ba2ff"},
	{ "pwd",     simple_parse,  "Working Directory",   "#6281ff", "#5975f7", "#8ba2ff"},
	{ "locate",  simple_parse,  "File Search",         "#6281ff", "#5975f7", "#8ba2ff"},
	{ "man",     man_parse,     "Manual Page",         "#ff8162", "#f77559", "#ffa28b"},
  { "du",      du_parse,      "Disk Usage",          "#6281ff", "#5975f7", "#8ba2ff"},
	{ NULL,     NULL,          NULL,                   NULL,      NULL,      NULL}
};


#if WINDOW_SHADOW_SHOWN
const double shadow_strip_l[5] = {
	.180, .478, .670, .831, .937
};

const double bottom_left_corner[5][5] = {
	{1.00, 1.00, 1.00, 1.00, 1.00},
	{.682, .898, .937, .968, .988},
	{.423, .800, .874, .937, .976},
	{.333, .682, .800, .898, .960},
	{.258, .584, .737, .866, .945},
};

const double bottom_right_corner[5][5] = {
	{.258, .584, .737, .866, .945},
	{.584, .682, .800, .898, .960},
	{.737, .800, .874, .937, .976},
	{.866, .898, .937, .968, .988},
	{.945, .960, .976, .988, .996} 
};

const double top_right_corner[5][5] = {
	{1.00, .686, .423, .333, .258},
	{1.00, .898, .803, .686, .584},
	{1.00, .937, .874, .800, .737},
	{1.00, .968, .937, .898, .866},
	{1.00, .988, .976, .960, .945},
};
#endif

/*****************************************************************************
 * CuiWindow                                                                 *
 *                                                                           *
 *****************************************************************************/
static void
cui_window_set_property (
		GObject      *object, 
		guint         property_id, 
		const GValue *value, 
		GParamSpec   *pspec) 
{
}

static void
cui_window_get_property (
		GObject      *object, 
		guint         property_id, 
		GValue       *value, 
		GParamSpec   *pspec)
{
	//CuiWindow *self = (CuiWindow *) object;
	g_value_set_string (value, "value");
}

static void
cui_window_class_init(
		gpointer g_class, 
		gpointer g_class_data)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);

	gobject_class->set_property = cui_window_set_property;
  	gobject_class->get_property = cui_window_get_property;
}


static void
cui_window_create_pango_layout(CuiWindow *window)
{
	Background *bg;
	PangoContext *context;
	PangoFontDescription *desc;
	PangoMatrix matrix = PANGO_MATRIX_INIT;;
	g_return_if_fail(CUI_IS_WINDOW(window));
	bg = cui_widget_get_background(CUI_WIDGET(window));
	/*
	 *
	 */
	gdk_color_parse("White", &window->titlebar_font_color);
	gdk_colormap_alloc_color(bg->colormap, 
			&window->titlebar_font_color, 
			FALSE, TRUE);

	context = gtk_widget_create_pango_context(
			GTK_WIDGET(window->text_view));
	pango_matrix_rotate(&matrix, 90.0);
	pango_context_set_matrix(context, &matrix);
	
	window->titlebar_layout = pango_layout_new(context);
	desc = pango_font_description_from_string(TITLEBAR_FONT);
  	pango_layout_set_font_description(window->titlebar_layout, desc);
	pango_layout_set_indent(window->titlebar_layout, 0);
	pango_layout_set_alignment(window->titlebar_layout, PANGO_ALIGN_LEFT);
	pango_layout_set_ellipsize(window->titlebar_layout, PANGO_ELLIPSIZE_END);
}

static void
cui_window_instance_init(
		GTypeInstance   *instance, 
		gpointer         g_class)
{
	CuiWindow *self = (CuiWindow *)instance;
	
	self->text_view = NULL;
	self->text_buffer = NULL;
	self->begin = NULL;
	self->end = NULL;
	self->parser_cursor = NULL;
	self->lexer_cursor = NULL; 
	
	self->command = g_string_new("");
	self->wd = g_string_new("");
	self->username = g_string_new("");
	self->hostname = g_string_new("");

	self->title = NULL;
	self->realized = FALSE;
	self->parsed_chars = 0;
	self->parsed_lines = 0;
	self->error_code = 0;

}

static gboolean
cui_window_expose_handler(CuiWindow *window,
		gint arg1,
		gpointer user_data)
{
	GdkRectangle frame_area;
	GdkRectangle titlebar_area;
	
	Background *bg;
	GdkColor *color, *hcolor, *lcolor;
	gint border = 1;
	gint n;
	
	g_return_val_if_fail(CUI_IS_WINDOW(window), FALSE);
	if (!window->realized) // || window->parsed_lines <= 0)
		return FALSE;
	bg = cui_widget_get_background(CUI_WIDGET(window));
	g_return_val_if_fail(bg != NULL, FALSE);	
	/*
	 * Calculating the areas in window coordinates.
	 */
	gtk_text_view_buffer_to_window_coords(bg->text_view, 
			GTK_TEXT_WINDOW_TEXT, 
			window->titlebar_area.x, 
			window->titlebar_area.y, 
			&titlebar_area.x, 
			&titlebar_area.y);
	titlebar_area.height = window->titlebar_area.height;
	titlebar_area.width = window->titlebar_area.width;
	gtk_text_view_buffer_to_window_coords(bg->text_view, 
			GTK_TEXT_WINDOW_TEXT, 
			window->frame_area.x, 
			window->frame_area.y, 
			&frame_area.x, 
			&frame_area.y);
	frame_area.height = window->frame_area.height;
	frame_area.width = window->frame_area.width;

	/*
	 * Searching for colors.
	 */
	color = cui_rectangle_get_color(CUI_RECTANGLE(window));
	hcolor = cui_rectangle_get_high_color(CUI_RECTANGLE(window));
	lcolor = cui_rectangle_get_low_color(CUI_RECTANGLE(window));
	g_return_val_if_fail(color != NULL && 
			hcolor != NULL &&
			lcolor != NULL, FALSE);

	/*
	 * Now drawing the frame of the window.
	 */
	if (bg_conf_get_bool("/settings/draw_window_frames", TRUE)) {
		gdk_gc_set_foreground(bg->gc, color);
		gdk_draw_rectangle(bg->window, bg->gc, FALSE, 
				frame_area.x, 
				frame_area.y, 
				frame_area.width, 
				frame_area.height);
		
		gdk_gc_set_foreground(bg->gc, lcolor);
		gdk_draw_rectangle(bg->window, bg->gc, FALSE, 
				frame_area.x + 1, 
				frame_area.y + 1, 
				frame_area.width - 2, 
				frame_area.height - 2);
		
		gdk_gc_set_foreground(bg->gc, hcolor);
		gdk_draw_rectangle(bg->window, bg->gc, FALSE, 
				frame_area.x + 2, 
				frame_area.y + 2, 
				frame_area.width - 4, 
				frame_area.height - 4);
		gdk_window_invalidate_rect(bg->window, &frame_area, TRUE);
	}

	/*
	 * Now drawing the titlebar area of the window.
	 */
	if (bg_conf_get_bool("/settings/title_text", TRUE)) {
		gdk_gc_set_foreground(bg->gc, color);
		gdk_draw_rectangle(bg->window, bg->gc, TRUE, 
				titlebar_area.x + border, 
				titlebar_area.y + border, 
				titlebar_area.width - 2 * border + 1, 
				titlebar_area.height - 2 * border + 1);
		gdk_gc_set_foreground(bg->gc, hcolor);
		for (n = 0; n < border; ++n) {
			gdk_draw_line(bg->window, bg->gc, 
					titlebar_area.x, 
					titlebar_area.y + n, 
					titlebar_area.x + 
						titlebar_area.width, 
					titlebar_area.y + n);
			gdk_draw_line(bg->window, bg->gc, 
					titlebar_area.x + n, 
					titlebar_area.y, 
					titlebar_area.x + n, 
					titlebar_area.y + 
						titlebar_area.height);
		}
		gdk_gc_set_foreground(bg->gc, lcolor);
		for (n = 0; n < border; ++n) {
			gdk_draw_line(bg->window, bg->gc, 
					titlebar_area.x + titlebar_area.width - n,
				      	titlebar_area.y + n + 1, 
				      	titlebar_area.x + titlebar_area.width - n, 
				      	titlebar_area.y + titlebar_area.height - 1);
			gdk_draw_line(bg->window, bg->gc,
				      	titlebar_area.x + n, titlebar_area.y + 
						titlebar_area.height - n, 
				      	titlebar_area.x + titlebar_area.width, 
				      	titlebar_area.y + titlebar_area.height - n);
		}
		/*
		 * Now writing the titlebar text.
		 */
		gdk_gc_set_foreground(bg->gc, &window->titlebar_font_color);
		pango_layout_set_width(window->titlebar_layout, 
				(titlebar_area.height - 10) * PANGO_SCALE);
		pango_layout_set_text(window->titlebar_layout,
				window->title, -1);
		gdk_draw_layout(GDK_DRAWABLE(bg->window), bg->gc,
				titlebar_area.x,
	                        titlebar_area.y + 10,
	                        window->titlebar_layout);
		gdk_window_invalidate_rect(bg->window, &titlebar_area, TRUE);
	}

	/*
	 * Drawing window shadow.
	 */
	if (bg_conf_get_bool("/settings/draw_window_shadow", TRUE)) {
		if (window->south_shadow != NULL)
			gdk_draw_pixbuf(bg->window,        /*GdkDrawable *drawable,*/
	        	                bg->gc,            /*GdkGC *gc,*/
	                	        window->south_shadow, /*GdkPixbuf *pixbuf,*/
	                        	0,                /*gint src_x,*/
		                        0,                /*gint src_y,*/ 
					0,                /*gint dest_x,*/
	                        	frame_area.y + frame_area.height + 1, /*gint dest_y,*/
	                        	-1,               /*gint width, */
	                        	-1,               /*gint height, */
	                        	GDK_RGB_DITHER_NONE, /*GdkRgbDither dither,*/
	                        	0,                /*gint x_dither,*/
	                        	0                 /*gint y_dither*/
					);
		if (window->east_shadow != NULL)
			gdk_draw_pixbuf(bg->window,        /*GdkDrawable *drawable,*/
	                	        bg->gc,            /*GdkGC *gc,*/
	                        	window->east_shadow, /*GdkPixbuf *pixbuf,*/
	                        	0,                /*gint src_x,*/
	                        	0,                /*gint src_y,*/
	                        	titlebar_area.width + frame_area.width, /*gint dest_x,*/
	                        	frame_area.y + 1, /*gint dest_y,*/
	                        	-1,                /*gint width, */
	                        	-1,                /*gint height, */
	                        	GDK_RGB_DITHER_NONE, /*GdkRgbDither dither,*/
	                        	0,                /*gint x_dither,*/
	                        	0                 /*gint y_dither*/
					);
	}
	return TRUE;
}
	
static void
cui_window_base_class_init(CuiWindowClass *klass)
{
	static gboolean initialized = FALSE;
	GClosure *class_closure;
	GType params[1];
	guint id;

	if (!initialized){
		/* interface signals creation*/
		class_closure = g_cclosure_new(
				G_CALLBACK(cui_window_expose_handler),
				NULL, /* user data */
				NULL  /* destroy data */
				);
		params[0] = G_TYPE_INT;
		/* here I'm */
		g_signal_override_class_closure(
				cui_rectangle_get_expose_signal_id(),
				CUI_WINDOW_TYPE,
                                class_closure);
		initialized = TRUE;
	}
}


GType 
cui_window_get_type(void)
{
	static GType type = 0;
	
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof(CuiWindowClass),
			(GBaseInitFunc) cui_window_base_class_init,
			NULL,
			(GClassInitFunc) cui_window_class_init,
			NULL,
			NULL,
			sizeof(CuiWindow),
			128,
			(GInstanceInitFunc) cui_window_instance_init 
		};
		
		type = g_type_register_static(CUI_RECTANGLE_TYPE, 
				"CuiWindow", 
				&info, 
				0);
	}
	return type;
}

/**
 * cui_frame_new:
 * @background: the background on which this window will appear
 * @returns: a new #CuiWindow
 *
 * Creates a new #CuiWindow widget. This function also registers the widget in
 * the backgrounds widget list.
 */
CuiWidget *
cui_window_new(Background *background)
{
	CuiWidget *widget;
	CuiWindow *window;
	g_assert(background != NULL);
	/*
	 * Creating the new object.
	 */
	widget = g_object_new(CUI_WINDOW_TYPE, NULL);
	window = CUI_WINDOW(widget);
	/*
	 * Setting a few important pointer.
	 */
	cui_widget_set_background(widget, background);
	window->text_view = background->text_view;
	window->text_buffer = background->text_buffer;
	/*
	 * Creatint PangoLayouts
	 */
	cui_window_create_pango_layout(window);
	window->south_shadow = NULL;
	window->east_shadow = NULL;
	return widget;
}

/**
 * cui_window_set_begin_mark:
 * @window: a #CuiWindow
 * @mark: the begin point as a #GtkTextMark
 *
 * Sets the start point of the window.
 */
void
cui_window_set_begin_mark(CuiWindow *window, GtkTextMark *mark)
{
	g_return_if_fail(CUI_IS_WINDOW(window));
	g_return_if_fail(GTK_IS_TEXT_MARK(mark));
	window->begin = mark;
}

/**
 * cui_window_set_end_mark:
 * @window: a #CuiWindow
 * @mark: the end point as a #GtkTextMark
 *
 * Sets the end point of the window.
 */
void
cui_window_set_end_mark(CuiWindow *window, GtkTextMark *mark)
{
	g_return_if_fail(CUI_IS_WINDOW(window));
	g_return_if_fail(GTK_IS_TEXT_MARK(mark));
	window->end = mark;
}

/**
 * cui_window_set_command:
 * @window: a #CuiWindow
 * @command: the command created this window.
 *
 * Sets the command (the full command line) of which created this window. We use
 * this facility to repeat the command if we want to refresh the window after a
 * procedure has been completed on it.
 */
void
cui_window_set_command(CuiWindow *window, GString *command)
{
#ifdef DEBUG
	g_message("%s(): Setting command to: '%s'", __func__, command->str);
#endif
	g_return_if_fail(CUI_IS_WINDOW(window));
	g_return_if_fail(command != NULL);
	window->command = g_string_assign(window->command, command->str);
}

/**
 * cui_window_set_working_directory:
 * @window: a #CuiWindow
 * @wd: the new working directory
 *
 * Sets the directory path in which this window was created.
 */
void
cui_window_set_working_directory(CuiWindow *window, GString *wd)
{
	g_return_if_fail(CUI_IS_WINDOW(window));
	g_return_if_fail(wd != NULL);
	window->wd = g_string_assign(window->wd, wd->str);
}

/**
 * cui_window_get_working_directory:
 * @window: a #CuiWindow
 * @returns: the working directory
 *
 * Returns the working directory of the window. This is a directory which was
 * the working directory when the window creator command was executed.
 */
const gchar *
cui_window_get_working_directory(CuiWindow *window)
{
	g_return_val_if_fail(CUI_IS_WINDOW(window), NULL);
	return window->wd->str;
}
	
/**
 * cui_window_apply_tag:
 * @window: the #CuiWindow
 * @tag: the name of the tag
 * @mbegin: the starting point of the text we highlight
 * @mend: the end point of the text we highlight
 *
 * Applies a text attribute tag in the window. A command parser can use this
 * function to create syntax highlights in the terminal.
 */
void 
cui_window_apply_tag(CuiWindow *window, 
		gchar *tag,
		GtkTextMark *mbegin,
		GtkTextMark *mend)
{
	GtkTextIter begin, end;
	g_return_if_fail(CUI_IS_WINDOW(window));
	g_return_if_fail(GTK_IS_TEXT_MARK(mbegin) &&
			GTK_IS_TEXT_MARK(mend));
	g_return_if_fail(tag != NULL);
	
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &begin, mbegin);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &end, mend);
	gtk_text_buffer_apply_tag_by_name(window->text_buffer, tag, 
			&begin, &end);
}

/*
 * A static function to calculate the bounding region of the window. The window
 * itself usually is much wider than this since the lines can be a lot shorter
 * than the terminal sindow.
 */
static void
calculate_bounding_region(CuiWindow *window)
{
	GdkRectangle b_rec, e_rec;
	GtkTextIter begin, end, last;
	
	g_return_if_fail(CUI_IS_WINDOW(window));
	
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &begin, 
			window->begin);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &end, 
			window->begin);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &last, 
			window->end);
	
	
	//gtk_text_iter_forward_to_line_end(&end);
	gtk_text_view_get_iter_location(window->text_view, &begin, &b_rec);
	gtk_text_view_get_iter_location(window->text_view, &end, &e_rec);
	gdk_rectangle_union(&b_rec, &e_rec, &window->b_region);
	while (gtk_text_iter_compare(&end, &last) <= 0){
		if (!gtk_text_iter_forward_to_line_end(&end))
			break;
		gtk_text_view_get_iter_location(window->text_view, 
				&end, &e_rec);
		
		gdk_rectangle_union(&window->b_region, &e_rec, 
				&window->b_region);
		gtk_text_iter_forward_line(&end);
	}

	//cui_widget_set_area(CUI_WIDGET(window), &window->b_region);
}

/**
 * cui_window_realize:
 * @window: the window to realize
 *
 * Realizes the window by calculating some data and creating some widgets.
 */
void
cui_window_realize(CuiWindow *window)
{
	GtkTextIter 	iter, start, end;
	CuiWidget 	*cui_widget;
	Background 	*background;
	gint 		retval = 0;
	gint 		n;
	gchar 		*text = NULL;
	gint 		width, height;
#ifdef DEBUG	
	g_message("%s(): start ", __func__);
#endif
	
	g_return_if_fail(CUI_IS_WINDOW(window));
	background = cui_widget_get_background(CUI_WIDGET(window));
	/*
	 * We left the begin mark at the place where is was when we
	 * started the command. The next line is the first line of the
	 * answer we have.
	 */
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &iter,
			window->begin);
	gtk_text_iter_forward_chars(&iter, 1);
	gtk_text_buffer_move_mark(window->text_buffer, window->begin, &iter);
	
	/*
	 * Let's see how many characters are in this window. 
	 */
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &start, window->begin);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &end, window->end);
	text = gtk_text_buffer_get_text(window->text_buffer, &start, &end, TRUE);
	n = g_utf8_strlen(text, -1);
	g_free(text);
	if (n <= 1)
		return;
	
	/*
	 * Let's create cursor showing the character we read.
	 */
	window->parser_cursor = gtk_text_buffer_create_mark(
			window->text_buffer, NULL, &iter, TRUE);
	window->lexer_cursor = gtk_text_buffer_create_mark(
			window->text_buffer, NULL, &iter, TRUE);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &iter,
			window->end);
	
	/*
	 * Let's calculate the bounding region of the window.
	 */
	calculate_bounding_region(window);
	/*
	 *
	 */
	g_return_if_fail(window->parse_tree != NULL);
	gchar *producer = parse_tree_get_producer_program(
			window->parse_tree);
	/*
	 * Well, this is the place where we start up out little parser
	 * to analize the command made this window... 
	 */
	g_return_if_fail(window->command != NULL);
	window_for_lex = window;
	n = 0;
	while (executors[n].name != NULL){
		if (strcmp(producer, executors[n].name) == 0){
			retval = executors[n].executor(window);
			cui_window_set_title_if_empty(window,
					executors[n].window_title);
			cui_rectangle_set_color(CUI_RECTANGLE(window), 
				executors[n].color,
				executors[n].lcolor,
				executors[n].hcolor);
			goto output_parsed;
		}
		n++;
	}
	g_warning("%s(): No parser for '%s' calling the general one.",
			__func__, 
			producer);
	retval = general_parse(window);
	cui_window_set_title_if_empty(window,  _("General Application"));
	cui_rectangle_set_color(CUI_RECTANGLE(window), 
			"#6281ff", "#5975f7", "#8ba2ff");
	
output_parsed:
	if (retval != 0 ||			   /* syntax error      */
			window->error_code != 0)   /* application error */
		cui_rectangle_set_color(CUI_RECTANGLE(window), 
				"#ed5959", "#ed4a4a", "#ed7979");
	
	/*
	 * Calculating the area where the frame of the window will appear. Well,
	 * we have to leave some place for the frame lines (3 pixel wide).
	 */
	GdkWindow *gdk_window;
	gdk_window = gtk_text_view_get_window(window->text_view,
			GTK_TEXT_WINDOW_WIDGET);
	gdk_drawable_get_size(GDK_DRAWABLE(gdk_window), &width, NULL);
	width = width - LEFT_MARGIN + 2;
	
	window->frame_area.x = LEFT_MARGIN - 3;
	window->frame_area.y = window->b_region.y - 3;
	window->frame_area.width = width - 5;
	window->frame_area.height = window->b_region.height + 6;

	window->titlebar_area.x = 0;
	window->titlebar_area.y = window->b_region.y - 3;
	window->titlebar_area.width = LEFT_MARGIN - 2;
	window->titlebar_area.height = window->b_region.height + 6;
	/*
	 * The full window area is used as an area for the window menu which 
	 * can be used to delete and detach the window.
	 */
	GtkWidget *gtk_widget;
	gtk_widget = create_window_menu();
	g_object_set_data(G_OBJECT(gtk_widget), "window", window);
	cui_widget = cui_popup_menu_new(gtk_widget);
	cui_widget_set_background(cui_widget, background);
	cui_widget_set_area_by_coord(cui_widget,
			0, window->b_region.y - 3,
			width + LEFT_MARGIN, window->b_region.height + 6);
	cui_widget_append_window_command(cui_widget, WindowRefresh, window);
	/*
	 * Creating shadow pixbuf
	 */
#if WINDOW_SHADOW_SHOWN
	gint rowstride;
	guchar *pixels, *p;
	gint i;
	gint j;
	gint index;
	
	width = window->titlebar_area.width + window->frame_area.width + 5;
	height = window->b_region.height + 6;
	
	window->south_shadow = gdk_pixbuf_get_from_drawable(
			NULL,               // GdkPixbuf *dest
			background->window, // GdkDrawable *src
			NULL,               // GdkColormap *colormap
			0, 		    // x
			window->frame_area.y + window->frame_area.height + 1,    // y
			0, 
			0, 
			width, 
			5);
	gdk_pixbuf_fill(window->south_shadow, 0xffffffff);
	
	rowstride = gdk_pixbuf_get_rowstride(window->south_shadow);
	pixels = gdk_pixbuf_get_pixels(window->south_shadow);
	for (i = 0; i < 5; i++) {
		p = pixels + (i * rowstride); 
		for (index = 0, j = 0; j < 3 * 5; j += 3, ++index) {
			p[j]     = (guchar) (p[j    ] * bottom_left_corner[index][i]); 
			p[j + 1] = (guchar) (p[j + 1] * bottom_left_corner[index][i]); 
			p[j + 2] = (guchar) (p[j + 2] * bottom_left_corner[index][i]); 
		}
		for (; j < 3 * (width - 5); j += 3) { 
			p[j]     = (guchar) (p[j    ] * bottom_right_corner[0][i]); 
			p[j + 1] = (guchar) (p[j + 1] * bottom_right_corner[0][i]); 
			p[j + 2] = (guchar) (p[j + 2] * bottom_right_corner[0][i]); 
		}
		for (index = 0; j < 3 * width; j += 3, ++index) {
			p[j]     = (guchar) (p[j    ] * bottom_right_corner[index][i]); 
			p[j + 1] = (guchar) (p[j + 1] * bottom_right_corner[index][i]); 
			p[j + 2] = (guchar) (p[j + 2] * bottom_right_corner[index][i]); 
		}
	}
	
	window->east_shadow = gdk_pixbuf_get_from_drawable(
			NULL,               // GdkPixbuf *dest
			background->window, // GdkDrawable *src
			NULL,               // GdkColormap *colormap
			window->titlebar_area.width + window->frame_area.width, //x
			window->frame_area.y,    // y
			0, 
			0, 
			5, 
			height);
	gdk_pixbuf_fill(window->east_shadow, 0xffffffff);
	
	rowstride = gdk_pixbuf_get_rowstride(window->east_shadow);
	pixels = gdk_pixbuf_get_pixels(window->east_shadow);
	for (i = 0; i < 5; i++) {
	    p = pixels + (i * rowstride);
	    for (j = 0, index = 0; j < 3 * 5; j += 3, ++index) { 
		    p[j    ] = (guchar) (p[j    ] * top_right_corner[index][i]); 
		    p[j + 1] = (guchar) (p[j + 1] * top_right_corner[index][i]); 
		    p[j + 2] = (guchar) (p[j + 2] * top_right_corner[index][i]);
	      }
	}
	for (; i < height; i++) {
	    p = pixels + (i * rowstride);
	    for (j = 0; j < 3 * 5; j += 3) { 
		    p[j    ] = (guchar) (p[j    ] * shadow_strip_l[j / 3]); 
		    p[j + 1] = (guchar) (p[j + 1] * shadow_strip_l[j / 3]); 
		    p[j + 2] = (guchar) (p[j + 2] * shadow_strip_l[j / 3]);
	      }
	  }
#endif
	/*
	 * Creating the dnd target area for the entire window.
	 */
	CuiFilename *fn;
	GtkTargetList *tlist;
	fn = cui_filename_new(".", window->wd->str);
	cui_filename_set_filetype_dir(fn);
	cui_widget = cui_rectangle_new();
	cui_widget_set_background(cui_widget, background);
	cui_widget_set_area_by_coord(cui_widget, 
		window->frame_area.x + 3,
		window->frame_area.y + 3,
		window->frame_area.width - 4,
		window->frame_area.height - 4);
	
	tlist = dnd_get_dir_target_list();
	cui_rectangle_set_target_list(CUI_RECTANGLE(cui_widget), tlist);
	cui_rectangle_set_target_object(CUI_RECTANGLE(cui_widget), 
			G_OBJECT(fn));
	cui_rectangle_set_is_dnd_target(CUI_RECTANGLE(cui_widget), TRUE);
	cui_widget_append_window_command(cui_widget, WindowRefresh, window);
	
	window->realized = TRUE;
}

/**
 * cui_window_get_token_marks:
 * @window: the embedded window
 * @token: the token was identified by the lexer and we want to read
 * @mbegin: the place where we put the start pointer, or NULL
 * @mend: the place where we put the end pointer, or NULL
 * @returns: TRUE if the token found
 *
 * Gets the text marks of the next lexical element in the window. A lexical
 * analyzer can use this function to find out the borders of a given lexical
 * element. If a lexical analizer uses this function then itm ust call for every
 * lexical element!
 */
gboolean
cui_window_get_token_marks(CuiWindow *window,
		char *token,
		GtkTextMark **mbegin,
		GtkTextMark **mend)
{
	gboolean     retval = TRUE;
	GtkTextIter  begin, end;
	gchar       *text;
#ifdef DEBUG
	g_message("%s(): start", __func__);
#endif
	g_return_val_if_fail(CUI_IS_WINDOW(window), FALSE);
	g_return_val_if_fail(window->lexer_cursor != NULL, FALSE);
		
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &begin, 
			window->lexer_cursor);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &end, 
			window->lexer_cursor);
	gtk_text_iter_forward_chars(&end, g_utf8_strlen(token, -1));
	text = gtk_text_buffer_get_text(window->text_buffer, &begin, &end, 
			FALSE);

	if (strcmp(token, text) != 0){
		g_warning(_("%s: Searching token '%s', found '%s'"), 
				__func__, token, text);
		retval = FALSE;
	}
	
	g_free(text);
	
	/*
	 * We send back the marks if we asked for it.
	 */
	if (mbegin != NULL)
		*mbegin = gtk_text_buffer_create_mark(
				window->text_buffer, NULL, &begin, FALSE);
	if (mend != NULL)
		*mend   = gtk_text_buffer_create_mark(
				window->text_buffer, NULL, &end, FALSE);
#if 0	
	gtk_text_buffer_delete_mark(window->text_buffer, window->lexer_cursor);
	window->lexer_cursor = gtk_text_buffer_create_mark(window->text_buffer, 
			NULL, &end, FALSE);
#endif	
	gtk_text_buffer_move_mark(window->text_buffer, 
			window->lexer_cursor,
			&end);
#ifdef DEBUG
	g_message("%s(): end", __func__);
#endif
	return retval;
}

/**
 * cui_window_get_char:
 * @window: the #CuiWindow 
 * @buf: the buffer the lexer made for us to store the characters
 * @max_size: the size of the buffer
 * @returns: the number of characters we stored in the buffer
 *
 * Reads the next character from the window. A lexical analyzer can use this
 * function to read characters from the program output.
 *
 * FIXME: This function have a few problems. First it use a global variable,
 * since I don't know how to write a multi-threaded parser with yacc/lex, 
 * second it sends back only one char and it cant waint for the characters to
 * arrive to the GtkTextBuffer.
 */
inline int 
cui_window_get_char(
		CuiWindow * window, 
		char *buf, 
		int max_size)
{
	GtkTextIter end_iter;
	GtkTextIter iter, iter1;
	gboolean    moved;
	gchar *text;

	g_assert(CUI_IS_WINDOW(window));
	
	/*
	 * Get the endpoint iter. This is the place where we have to change if
	 * we want a continuous reading scanner.
	 */
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &end_iter,
					 window->end);
	/*
	 * Get the next char from the textbuffer.
	 */	
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &iter,
					 window->parser_cursor);

	if (gtk_text_iter_compare(&iter, &end_iter) > 0){
		return 0;
	}
	
	/*
	 * Getting the text.
	 */
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &iter1,
					 window->parser_cursor);
	moved = gtk_text_iter_forward_char(&iter1);
#ifdef DEBUG
	if (!moved)
		g_message("%s(): iter1 is at the end of buffer?", __func__);
#endif

	text = gtk_text_buffer_get_text(window->text_buffer, &iter, &iter1,
				     FALSE);
	/*
	 * Moving the cursor
	 */
	gtk_text_buffer_move_mark(window->text_buffer,
				  window->parser_cursor, &iter1);
	/*
	 * Create a new mark showing the lexer where this character came from.
	 */
	strcpy(buf, text);
	g_free(text);

	window->parsed_chars++;
	if (buf[0] == '\n')
		window->parsed_lines++;
	
#ifdef DEBUG
	g_message("%s(): got string: '%s' %d", __func__, buf, buf[0]);
#endif
	return strlen(buf);
}

/**
 * cui_window_set_title:
 *
 *
 */
void
cui_window_set_title(CuiWindow *window, gchar *title)
{
	g_return_if_fail(CUI_IS_WINDOW(window));
	if (title == NULL)
		title = "";

	g_free(window->title);
	window->title = g_strdup(title);
}

/**
 * cui_window_get_title:
 * @window: the #CuiWindow
 * @returns: the window title
 * 
 * Returns the window title of a #Cuiwindow. The return string should not be
 * changed or freed.
 */
gchar *
cui_window_get_title(CuiWindow *window)
{
	g_return_if_fail(CUI_IS_WINDOW(window));
	return window->title;
}

	
/**
 * cui_window_set_title:
 * @window: a #CuiWindow
 * @title: a new window title
 *
 * Sets the window title of the window if it is empty, otherwise do nothing.
 *
 */
void
cui_window_set_title_if_empty(CuiWindow *window, gchar *title)
{
	g_return_if_fail(CUI_IS_WINDOW(window));
	if (window->title != NULL)
		return;
	window->title = g_strdup(title);
}

/**
 * cui_window_set_parse_tree:
 * @window: a #CuiWindow
 * @parse_tree: a shell parse tree
 *
 * Sets the parse tree for the window. The window uses the parse tree to decide
 * what to do with the programs output.
 */
void
cui_window_set_parse_tree(CuiWindow *window, GNode *parse_tree)
{
	g_return_if_fail(CUI_IS_WINDOW(window));
	window->parse_tree = parse_tree;
}
