/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: cuiwindow.h,v 1.1 2006/02/24 17:32:14 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: cuiwindow.h,v $
 * Revision 1.1  2006/02/24 17:32:14  pipas
 * Initial revision
 *
 */
#ifndef CUIWINDOW_H
#define CUIWINDOW_H

#include <gnome.h>
#include <sys/timeb.h>
#include "cuiterm.h"
#include "widgets.h"

/******************************************************************************
 * Instantiable classed type: CuiFrame                                        *
 *                                                                            *
 ******************************************************************************/
#define CUI_WINDOW_TYPE (cui_window_get_type ())
#define CUI_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_WINDOW_TYPE, CuiWindow))
#define CUI_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_WINDOW_TYPE))
#define CUI_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_WINDOW_TYPE, CuiWindowClass))

/**
 * CuiWindow:
 * @begin: the starting point of the window is at the command line
 * @end: the end point of the window
 * @parser_cursor: the point where the parser reads the text (FIXME: do we need this?)
 * @lexer_cursor: the point where the lexer reads the text
 * @command: the full command line created this command
 * @parse_tree: A tree created by the bash parser. Every element is a
 * #CuiLexElement.
 * @wd: the working directory part from the prompt
 * @username: the user name part from the cursor
 * @hostname: the hostname part from the cursor
 */
typedef struct _CuiWindow CuiWindow;
struct _CuiWindow {
	CuiRectangle 	parent;
	gboolean	realized;
	GdkRectangle	frame_area;
	GdkRectangle	titlebar_area;
	PangoLayout 	*titlebar_layout;
	GdkColor	titlebar_font_color;
	gchar 		*title;
	gint 		error_code;
	gint 		parsed_chars;
	gint 		parsed_lines;
	GtkTextView 	*text_view;
	GtkTextBuffer 	*text_buffer;
	GdkRectangle 	b_region;
	GdkPixbuf       *east_shadow;
	GdkPixbuf       *south_shadow;
	/*< public >*/
	GtkTextMark 	*begin;
	GtkTextMark 	*end;
	GtkTextMark 	*parser_cursor;    /* wandering cursor */
	GtkTextMark 	*lexer_cursor;     /* */
	GString 	*command;
	GNode 		*parse_tree;
	GString 	*wd;
	GString 	*username;
	GString 	*hostname;
};

typedef struct {
	CuiRectangleClass parent_class;
	/* public */
} CuiWindowClass;

GType cui_window_get_type(void);
CuiWidget *cui_window_new(Background *background);
void cui_window_set_begin_mark(CuiWindow *window, GtkTextMark *mark);
void cui_window_set_end_mark(CuiWindow *window, GtkTextMark *mark);
void cui_window_set_command(CuiWindow *window, GString *command);
void cui_window_set_working_directory(CuiWindow *window, GString *wd);
const gchar *cui_window_get_working_directory(CuiWindow *window);
void cui_window_apply_tag(CuiWindow *window, 
		gchar *tag,
		GtkTextMark *mbegin,
		GtkTextMark *mend);
void cui_window_realize(CuiWindow *window);
gboolean cui_window_get_token_marks(CuiWindow *window,
		char *token,
		GtkTextMark **mbegin,
		GtkTextMark **mend);
inline int cui_window_get_char(CuiWindow * window, char *buf, int max_size);
void cui_window_set_title(CuiWindow *window, gchar *title);
void cui_window_set_title_if_empty(CuiWindow *window, gchar *title);
void cui_window_set_parse_tree(CuiWindow *window, GNode *parse_tree);
gchar *cui_window_get_title(CuiWindow *window);


#endif
