%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: df_parser.y,v 1.1 2006/02/24 08:22:52 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: df_parser.y,v $
 * Revision 1.1  2006/02/24 08:22:52  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "interface.h"
#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "widgets.h"
#include "cuiwindow.h"

int df_wrap(void);
int df_error(CuiWindow *window, char *);

extern CuiWindow *window_for_lex;
static CuiLexElement fll;
static CuiFilename *fn;
%}

/*%debug*/
%parse-param {CuiWindow *window}
%union {
	CuiLexElement sval;
};

%token <sval> HEADER
%token <sval> BLOCK_DEVICE MOUNT_POINT NUMBER PERCENT FILESYSTEM
%%

lines: lines line
     | line
     ;

line: HEADER
    | BLOCK_DEVICE filesystem NUMBER NUMBER NUMBER PERCENT MOUNT_POINT 
         {
	   /* filename, menu and action area*/
	   CuiFilename *fn;
	   Background *background;
	   CuiWidget *widget;
	   GtkWidget *menu;
	   background = cui_widget_get_background(CUI_WIDGET(window));
	   fn = cui_filename_new($7.text, window->wd->str); 
           cui_filename_set_filetype_dir(fn);
	   cui_action_area_new_for_filename(window, background, fn,
				$7.begin, $7.end);
	   cui_file_menu_new_for_filename(window, background, fn,
	   			$7.begin, $7.end);
	   /*
	    * The size rectangle.
	    */
	   widget = cui_rectangle_emboss_new();
	   cui_widget_set_background(widget, background);
	   cui_widget_set_area_by_marks(widget, $1.begin, $6.end);
	   if ($6.double_val < 0.9)
	   	cui_rectangle_set_color(CUI_RECTANGLE(widget), 
	   		"#87f387",
	   		"#44f344",
			"#bef3be"); 
	   else
	   	cui_rectangle_set_color(CUI_RECTANGLE(widget), 
	   		"#f3acac",
	   		"#f37777",
			"#f3cbcb"); 
	   cui_widget_set_width_by_percent(widget, $6.double_val);
	   /*
	    * Menu for the block device.
	    */
	   menu = create_block_device_menu();
	   g_object_set_data(G_OBJECT(menu), "filename", fn);
	   widget = cui_popup_menu_new(menu);
	   cui_widget_set_background(widget, background);
	   cui_widget_set_area_by_marks(widget, $1.begin, $1.end);
         }
    | error { window->error_code++; }

    ;

filesystem:
	  | FILESYSTEM
	  ;
%%

int df_error(CuiWindow *window, char *c)
{
	gchar *error_message;
	
	error_message = g_strdup_printf("Disk Usage Error: %s", c);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return 1;
}

int df_wrap(void)
{
	return 1;
}

