/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: dirhistory.c,v 1.1 2006/02/24 17:20:33 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: dirhistory.c,v $
 * Revision 1.1  2006/02/24 17:20:33  pipas
 * Initial revision
 *
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

/****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/
#include "support.h"
#include "interface.h"
#include "terminal.h"
#include "dirhistory.h"

static GtkComboBox *combo_box = NULL;
static GtkListStore *list_store = NULL;

static GList *dirhist = NULL;
static GList *dirhist_actual = NULL;

static gboolean from_history = FALSE;

static inline void
_set_buttons(void)
{
	GtkWidget *widget;
	GString *wd;
	
#ifdef DEBUG
	printf("%d(): dirhistory length.\n", g_list_length(dirhist));
	fflush(stdout);
#endif

	if (combo_box == NULL)
		return;

	widget = lookup_widget(GTK_WIDGET(combo_box), "dirback_toolbutton");
	if (dirhist_actual == NULL ||
			dirhist_actual->prev == NULL)
		gtk_widget_set_sensitive(widget, FALSE);
	else
		gtk_widget_set_sensitive(widget, TRUE);

	widget = lookup_widget(GTK_WIDGET(combo_box), "dirforward_toolbutton");
	if (dirhist_actual == NULL ||
			dirhist_actual->next == NULL)
		gtk_widget_set_sensitive(widget, FALSE);
	else
		gtk_widget_set_sensitive(widget, TRUE);
}

/***************************************************************************
 * Utility functions for the directory history of the GUI.                 *
 *                                                                         *
 **************************************************************************/
void 
dirhistory_directory_changed(GString *dir_nam)
{
	GtkWidget *widget;
	GtkTreeIter iter;
	gchar *name;
	gint n;	

	/*
	 * FIXME: this function can be called before the list_store created.
	 */
	if (list_store == NULL)
		return;
	/*
	 * Let's find if the directory name is already in the
	 * combobox.
	 */
	n = 0;
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list_store), &iter);
	while (1){
		gtk_tree_model_get(GTK_TREE_MODEL(list_store), 
				&iter, 0, &name, -1);
		if (strcmp(dir_nam->str, name) == 0)
			break;
		g_free(name);
		name = NULL;
		++n;
		if ( !gtk_tree_model_iter_next(
					GTK_TREE_MODEL(list_store), 
					&iter) )
			break;
	}
	/*
	 * If we could not find it we insert it.
	 */
	if (name == NULL){
		gtk_list_store_append(list_store, &iter);
		gtk_list_store_set(list_store, &iter, 
				0, dir_nam->str, 
				-1);
		gtk_combo_box_set_active(combo_box, n);	
	}else{
		/*
		 * Otherwise we select the existing item of the combobox.
		 */
		gtk_combo_box_set_active(combo_box, n);	
	}

	_set_buttons();
	if (from_history){
		from_history = FALSE;
		return;
	}

	dirhist = g_list_append(dirhist, g_strdup(dir_nam->str));
	dirhist_actual = g_list_last(dirhist);
	_set_buttons();
}


/***************************************************************************
 * Callback functions for the directory history of the GUI.                *
 *                                                                         *
 **************************************************************************/

void
on_combobox_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkTreeIter iter;

	combo_box = GTK_COMBO_BOX(widget);
	/*
	 * Creating a new treestore with the / directory
	 */
	list_store = gtk_list_store_new(1, G_TYPE_STRING);
	gtk_list_store_append(list_store, &iter);
	gtk_list_store_set(list_store, &iter, 
			0, "/", 
			-1);
	/*
	 * Setting up the combobox
	 */
	gtk_combo_box_set_model(combo_box, GTK_TREE_MODEL(list_store));
	gtk_combo_box_entry_set_text_column((GtkComboBoxEntry *)combo_box, 0);
	gtk_combo_box_set_active(combo_box, 0);	
}


void
on_combobox_changed                    (GtkWidget *combobox,
                                        gpointer   user_data)
{
	GtkEntry *entry = GTK_ENTRY(GTK_BIN(combo_box)->child);
	GtkWidget *widget = lookup_widget(combobox, "go_toolbutton");
	const gchar *text = gtk_entry_get_text(entry);
	
	gtk_widget_set_sensitive(widget, TRUE);
}


void
on_dirback_toolbutton_clicked          (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *text_view;
	Terminal *t;
	gchar *command;
	
	g_assert(dirhist_actual != NULL);
	g_assert(dirhist_actual->prev != NULL);

	text_view = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(text_view), "terminal");
	g_assert(t != NULL);
	
	dirhist_actual = dirhist_actual->prev;
	
	from_history = TRUE;
	command = g_strdup_printf("cd %s", (gchar *)dirhist_actual->data);
	terminal_send_command(t, command, TRUE);
	g_free(command);
	
	command = g_strdup_printf("ls%s", 
			bg_conf_get_default_ls_options(FALSE));
	terminal_send_command(t, command, TRUE);
	g_free(command);

	_set_buttons();
}


void
on_dirforward_toolbutton_clicked        (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *text_view;
	Terminal *t;
	gchar *command;
	
	g_assert(dirhist_actual != NULL);
	g_assert(dirhist_actual->next != NULL);

	text_view = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(text_view), "terminal");
	g_assert(t != NULL);
	
	dirhist_actual = dirhist_actual->next;

	from_history = TRUE;
	command = g_strdup_printf("cd %s", (gchar *)dirhist_actual->data);
	terminal_send_command(t, command, TRUE);
	g_free(command);
	
	command = g_strdup_printf("ls%s", 
			bg_conf_get_default_ls_options(FALSE));
	terminal_send_command(t, command, TRUE);
	g_free(command);
	
	_set_buttons();
}


void
on_go_toolbutton_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkEntry *entry = GTK_ENTRY(GTK_BIN(combo_box)->child);
	const gchar *text = gtk_entry_get_text(entry);
	GtkWidget *text_view;
	Terminal *t;
	gchar *command;

	text_view = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(text_view), "terminal");
	g_assert(t != NULL);

	command = g_strdup_printf("cd %s", text);
	terminal_send_command(t, command, TRUE);
	g_free(command);
	
	command = g_strdup_printf("ls%s", 
			bg_conf_get_default_ls_options(FALSE));
	terminal_send_command(t, command, TRUE);
	g_free(command);
	
	_set_buttons();
}
