%{
/*
 * $Id: du_lexer.l,v 1.4 2006/05/27 17:11:28 skinner Exp skinner $
 *
 * $Log: du_lexer.l,v $
 * Revision 1.4  2006/05/27 17:11:28  skinner
 * Rules for human readable filesizes implemented
 *
 * Revision 1.3  2006/05/27 14:38:32  skinner
 * Fixed spaces and underscore problem in file or directory names.
 *
 * Revision 1.2  2006/05/27 14:02:25  skinner
 * Need to fix the lexical analysis for spaces in file or dir names.
 *
 * Revision 1.1  2006/05/24 14:53:51  skinner
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef ALLDEBUG
# define DU_DEBUG
#endif

/*#define DU_DEBUG*/

#include <gnome.h>
#include <string.h>
#include "ls_parser_c.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "du_parser.h"

#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%option yylineno

%s DEFAULT ALL

%%
  cui_window_set_title(window_for_lex, _("Disk Usage"));
  if (window_for_lex->parsed_chars == 0)
    if (parse_tree_check_option(terminal_for_lex->parse_tree, "du", "a"))
      BEGIN(ALL);
    else
      BEGIN(DEFAULT);

<ALL,DEFAULT>[ \t\n\r]+ {
  /* skipping white chars to the next token */
  cui_window_get_token_marks(window_for_lex,
    yytext, NULL, NULL);
}

<ALL,DEFAULT>[0-9]+ {
  /* size */
#ifdef DU_DEBUG
  g_message("%s: SIZE: '%s'", __func__, yytext);
#endif
  du_lval.sval.text = strdup(yytext);
  du_lval.sval.double_val = atof(yytext);
  cui_window_get_token_marks(window_for_lex,
    du_lval.sval.text,
    &du_lval.sval.begin,
    &du_lval.sval.end);
  return SIZE;
}

<ALL,DEFAULT>[0-9.,]+[Gg] {
  du_lval.sval.text = strdup(yytext);
  du_lval.sval.double_val = g_ascii_strtod(yytext,NULL);
  du_lval.sval.double_val *= 1024 * 1024 * 1024;
  cui_window_get_token_marks(window_for_lex,
    du_lval.sval.text,
    &du_lval.sval.begin,
    &du_lval.sval.end);
  return SIZE;
}

<ALL,DEFAULT>[0-9.,]+[Mm] {
  du_lval.sval.text = strdup(yytext);
  du_lval.sval.double_val = g_ascii_strtod(yytext,NULL);
  du_lval.sval.double_val *= 1024 * 1024;
  cui_window_get_token_marks(window_for_lex,
    du_lval.sval.text,
    &du_lval.sval.begin,
    &du_lval.sval.end);
  return SIZE;
}

<ALL,DEFAULT>[0-9.,]+[Kk] {
  du_lval.sval.text = strdup(yytext);
  du_lval.sval.double_val = g_ascii_strtod(yytext,NULL);
  du_lval.sval.double_val *= 1024;
  cui_window_get_token_marks(window_for_lex,
    du_lval.sval.text,
    &du_lval.sval.begin,
    &du_lval.sval.end);
  return SIZE;
}

<ALL,DEFAULT>\.|total {
  /* total space used */
  du_lval.sval.text = strdup(yytext);
  cui_window_get_token_marks(window_for_lex,
    du_lval.sval.text,
    &du_lval.sval.begin,
    &du_lval.sval.end);
  return TOTAL;
}

<ALL,DEFAULT>[-\.,\/_a-zA-Z0-9 áéíöüűőúÁÉÍÖÜÓŐÚŰ]+ {
  du_lval.sval.text = strdup(yytext);
  cui_window_get_token_marks(window_for_lex,
    du_lval.sval.text,
    &du_lval.sval.begin,
    &du_lval.sval.end);
  return FILENAME;
}

<<EOF>> {
  return END_OF_WINDOW;
}

%%
