%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "fdisk_parser.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"

/* #define DEBUG */

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))

%}

%option yylineno

FS      [ \t]+
NUMBER  [0-9\.]+
%%
	cui_window_set_title(window_for_lex, _("Partition Table"));

"Disk" {
	#ifdef DEBUG
	g_message("%s(): DISK: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return DISK;
}

"bytes" {
	#ifdef DEBUG
	g_message("%s(): BYTES: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return BYTES;
}

"heads" {
	#ifdef DEBUG
	g_message("%s(): DISK: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return HEADS;
}

"sectors/track" {
	#ifdef DEBUG
	g_message("%s(): DISK: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return ST;
}

"cylinders" {
	#ifdef DEBUG
	g_message("%s(): DISK: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return CYLINDERS;
}

^"Units"[^\n]+ {
	#ifdef DEBUG
	g_message("%s(): COMMENT: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return COMMENT;
}

^[ \t]*"Device"[ \t]*"Boot"[^\n]+ {
	#ifdef DEBUG
	g_message("%s(): HEADER: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return HEADER;
}

"*" {
	#ifdef DEBUG
	g_message("%s(): ACTIVE: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "important", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return ACTIVE;
}




[GM]"B" {
	#ifdef DEBUG
	g_message("%s(): UNIT: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	/*
	cui_window_apply_tag(window_for_lex, "unimportant", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end);
	*/
	return UNIT;
}

"/"[a-z0-9/]+ {
	#ifdef DEBUG
	g_message("%s(): DEVICE: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "block_device", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	return DEVICE;
}

{NUMBER}[+]? {
	#ifdef DEBUG
	g_message("%s(): NUMBER: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	fdisk_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	/*cui_window_apply_tag(window_for_lex, "Block_device", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); */
	return NUMBER;
}

[:,] {
	#ifdef DEBUG
	g_message("%s(): Separator: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	return yytext[0];
}

[A-Z][a-zA-Z0-9]+ {
	#ifdef DEBUG
	g_message("%s(): SYSTEM: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
	/*
	cui_window_apply_tag(window_for_lex, "Header", 
		fdisk_lval.sval.begin, 
		fdisk_lval.sval.end); 
	*/
	return SYSTEM;
}


{FS} {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

\n+ {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

. {
	#ifdef DEBUG
	g_message("%s(): Unknown character: '%s'", __func__, yytext);
	#endif
	fdisk_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		fdisk_lval.sval.text,
		&fdisk_lval.sval.begin,
		&fdisk_lval.sval.end);
}

%%
