%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "getfacl_parser.h"

void 
cui_action_area_new_for_acl_user_rights(
		CuiWindow   *window,
		Background  *background,
		gchar       *username,
		gchar       *rights,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	CuiWidget *widget;	
	GtkWidget *appbar;
	gchar *tooltip;
	GtkTextIter iter;
	gchar *filename;
	gchar *active_tag;
	gboolean read_right, write_right, execute_right;
	gchar *acl;
#ifdef DEBUG
	g_message("%s(): starting on user '%s'", __func__, username);
#endif
	/*
	 * Checking and collecting information.
	 */
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	filename = cui_filename_get_filename(file);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, &iter, begin);
	/*
  	 *
	 */
	read_right = rights[0] == 'r';
	write_right = rights[1] == 'w';
	execute_right = rights[2] == 'x';
	/*
	 * The first field: the read right for the given user.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (read_right) {
		tooltip = g_strdup_printf(
				_("Read file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		active_tag = "yes_active";
		acl = g_strdup_printf( "u:%s:-%c%c", username,
			write_right ? 'w' : '-',
			execute_right ? 'x' : '-');
			
	} else {
		tooltip = g_strdup_printf(
				_("Read file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		active_tag = "no_active";	
		acl = g_strdup_printf( "u:%s:r%c%c", username,
			write_right ? 'w' : '-',
			execute_right ? 'x' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * The next field: the write right for the given user.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (write_right) {
		tooltip = g_strdup_printf(
				_("Write file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		active_tag = "yes_active";
		acl = g_strdup_printf( "u:%s:%c-%c", username,
			read_right ? 'r' : '-',
			execute_right ? 'x' : '-');
			
	} else {
		tooltip = g_strdup_printf(
				_("Write file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		active_tag = "no_active";	
		acl = g_strdup_printf( "u:%s:%cw%c", username,
			read_right ? 'r' : '-',
			execute_right ? 'x' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * The next field: the execute right for the given user.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (execute_right) {
		tooltip = g_strdup_printf(
				_("Execute file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		active_tag = "yes_active";
		acl = g_strdup_printf( "u:%s:%c%c-", username,
			read_right ? 'r' : '-',
			write_right ? 'w' : '-');
			
	} else {
		tooltip = g_strdup_printf(
				_("Execute file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		active_tag = "no_active";	
		acl = g_strdup_printf( "u:%s:%c%cx", username,
			read_right ? 'r' : '-',
			write_right ? 'w' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
}


void 
cui_action_area_new_for_acl_group_rights(
		CuiWindow   *window,
		Background  *background,
		gchar       *username,
		gchar       *rights,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	CuiWidget *widget;	
	GtkWidget *appbar;
	gchar *tooltip;
	GtkTextIter iter;
	gchar *filename;
	gchar *active_tag;
	gboolean read_right, write_right, execute_right;
	gchar *acl;
#ifdef DEBUG
	g_message("%s(): starting on group '%s'", __func__, username);
#endif
	/*
	 * Checking and collecting information.
	 */
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	filename = cui_filename_get_filename(file);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, &iter, begin);
	/*
  	 *
	 */
	read_right = rights[0] == 'r';
	write_right = rights[1] == 'w';
	execute_right = rights[2] == 'x';
	/*
	 * The first field: the read right for the given user.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (read_right) {
		tooltip = g_strdup_printf(
			_("Read file '%s' for members of %s is granted. "
			"Double click to revoke it."),
				filename, username);
		active_tag = "yes_active";
		acl = g_strdup_printf( "g:%s:-%c%c", username,
			write_right ? 'w' : '-',
			execute_right ? 'x' : '-');
			
	} else {
		tooltip = g_strdup_printf(
			_("Read file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
				filename, username);
		active_tag = "no_active";	
		acl = g_strdup_printf( "g:%s:r%c%c", username,
			write_right ? 'w' : '-',
			execute_right ? 'x' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * The next field: the write right for the given user.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (write_right) {
		tooltip = g_strdup_printf(
			_("Write file '%s' for members of %s is granted. "
			"Double click to revoke it."),
				filename, username);
		active_tag = "yes_active";
		acl = g_strdup_printf( "g:%s:%c-%c", username,
			read_right ? 'r' : '-',
			execute_right ? 'x' : '-');
			
	} else {
		tooltip = g_strdup_printf(
			_("Write file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
				filename, username);
		active_tag = "no_active";	
		acl = g_strdup_printf( "g:%s:%cw%c", username,
			read_right ? 'r' : '-',
			execute_right ? 'x' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * The next field: the execute right for the given user.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (execute_right) {
		tooltip = g_strdup_printf(
			_("Execute file '%s' for members of %s is granted. "
			"Double click to revoke it."),
				filename, username);
		active_tag = "yes_active";
		acl = g_strdup_printf( "g:%s:%c%c-", username,
			read_right ? 'r' : '-',
			write_right ? 'w' : '-');
			
	} else {
		tooltip = g_strdup_printf(
			_("Execute file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
				filename, username);
		active_tag = "no_active";	
		acl = g_strdup_printf( "g:%s:%c%cx", username,
			read_right ? 'r' : '-',
			write_right ? 'w' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
}

void 
cui_action_area_new_for_acl_mask_rights(
		CuiWindow   *window,
		Background  *background,
		gchar       *rights,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	CuiWidget *widget;	
	GtkWidget *appbar;
	gchar *tooltip;
	GtkTextIter iter;
	gchar *filename;
	gchar *active_tag;
	gboolean read_right, write_right, execute_right;
	gchar *acl;
#ifdef DEBUG
	g_message("%s(): starting on mask.", __func__);
#endif
	/*
	 * Checking and collecting information.
	 */
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	filename = cui_filename_get_filename(file);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, &iter, begin);
	/*
  	 *
	 */
	read_right = rights[0] == 'r';
	write_right = rights[1] == 'w';
	execute_right = rights[2] == 'x';
	/*
	 * The first field: the read right for the given user.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (read_right) {
		tooltip = g_strdup_printf(
			_("Read file '%s' enabled where available. "
			"Double click to disable it."),
				filename);
		active_tag = "yes_active";
		acl = g_strdup_printf( "m::-%c%c",
			write_right ? 'w' : '-',
			execute_right ? 'x' : '-');
			
	} else {
		tooltip = g_strdup_printf(
			_("Read file '%s' is forbidden by mask. "
			"Double click to enable it."),
				filename);
		active_tag = "no_active";	
		acl = g_strdup_printf( "m::r%c%c", 
			write_right ? 'w' : '-',
			execute_right ? 'x' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * The next field: the write right for the given user.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (write_right) {
		tooltip = g_strdup_printf(
			_("Write file '%s' enabled where available. "
			"Double click to disable it."),
				filename);
		active_tag = "yes_active";
		acl = g_strdup_printf( "m::%c-%c",
			read_right ? 'r' : '-',
			execute_right ? 'x' : '-');
			
	} else {
		tooltip = g_strdup_printf(
			_("Write file '%s' is forbidden by mask. "
			"Double click to enable it."),
				filename);
		active_tag = "no_active";	
		acl = g_strdup_printf( "m::%cw%c",
			read_right ? 'r' : '-',
			execute_right ? 'x' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * The next field: the execute right for the given user.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (execute_right) {
		tooltip = g_strdup_printf(
			_("Execute file '%s' enable where available. "
			"Double click to disable it."),
				filename);
		active_tag = "yes_active";
		acl = g_strdup_printf( "m::%c%c-", 
			read_right ? 'r' : '-',
			write_right ? 'w' : '-');
			
	} else {
		tooltip = g_strdup_printf(
			_("Execute file '%s' is forbidden by mask. "
			"Double click to enable it."),
				filename);
		active_tag = "no_active";	
		acl = g_strdup_printf( "m::%c%cx", 
			read_right ? 'r' : '-',
			write_right ? 'w' : '-');
	}
	cui_widget_append_file_command_with_arg(widget, 
			FileChangeACL, file, acl);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
}

extern CuiWindow   *window_for_lex;

static CuiFilename *fn = NULL;
static Background  *background = NULL;
static CuiWidget   *widget = NULL;

%}

%parse-param {CuiWindow *window}
%union { 
	CuiLexElement sval;
}
 
%token <sval> ERROR_MESSAGE
%token <sval> FILE_LINE FILENAME
%token <sval> OWNER_LINE OWNERNAME
%token <sval> GROUPOWNER_LINE GROUPOWNERNAME
%token <sval> OWNER_RIGHTS_LINE GROUPOWNER_RIGHTS_LINE OTHER_RIGHTS_LINE
%token <sval> MASK_RIGHTS_LINE
%token <sval> USER_RIGHTS_LINE GROUP_RIGHTS_LINE USERNAME
%token <sval> GROUPNAME
%token <sval> RIGHTS
%%
lines: lines line
     | line
     ;

line: ERROR_MESSAGE      { g_warning("%s(): Unimplemented.", __func__);}
    | error              { window->error_code++; }
    | FILE_LINE FILENAME { 

		background = cui_widget_get_background(CUI_WIDGET(window));
    	   	fn = cui_filename_new($2.text, window->wd->str); 
           	//cui_filename_set_mode_string(fn, $2.text);
	  	//cui_filename_set_owner_and_group(fn, $4.text, $6.text);
   		
		/* Filename action area */
		cui_action_area_new_for_filename(window,
				background, fn,
				$2.begin, $2.end);
	   	cui_file_menu_new_for_filename(window, background, fn,
	   			$2.begin, $2.end);
    	}
    | OWNER_LINE OWNERNAME {
    		/* Owner action area. */
		cui_action_area_new_for_owner( window, background, fn,
			$2.begin, $2.end);
		/* Owner entry. */
		widget = cui_entry_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget, $2.begin, $2.end);
		cui_entry_set_default_text(CUI_ENTRY(widget), $2.text);
		cui_widget_append_file_command(widget, FileChangeOwner, fn);
		/* User menu on the owner */
	   	widget = cui_user_menu_new($2.text);
	   	cui_widget_set_background(widget, background);
	   	cui_widget_set_area_by_marks(widget, $2.begin, $2.end);
        }
    | GROUPOWNER_LINE GROUPOWNERNAME {
		/*
		 * The group owner.
		 */
		cui_action_area_new_for_group(window, background, fn,
			$2.begin, $2.end);
		/* Group owner entry */
		widget = cui_entry_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget, $2.begin, $2.end);
		cui_entry_set_default_text(CUI_ENTRY(widget), $2.text);
		cui_widget_append_file_command(widget, FileChangeGroup, fn);
        }
    | OWNER_RIGHTS_LINE RIGHTS {
		cui_filename_set_mode_string_owner(fn, $2.text);
	   	cui_action_area_new_for_mode_owner(window, background, 
				fn, $2.begin, $2.end);
        }
    | GROUPOWNER_RIGHTS_LINE RIGHTS {
		cui_filename_set_mode_string_group(fn, $2.text);
	   	cui_action_area_new_for_mode_group(window, background, 
				fn, $2.begin, $2.end);
        }
    | OTHER_RIGHTS_LINE RIGHTS {
		cui_filename_set_mode_string_others(fn, $2.text);
	   	cui_action_area_new_for_mode_others(window, background, 
				fn, $2.begin, $2.end);
        }
    | USER_RIGHTS_LINE USERNAME RIGHTS {
		cui_action_area_new_for_acl_user_rights(window, background,
				$2.text, $3.text,
				fn, $3.begin, $3.end);
       }
    | GROUP_RIGHTS_LINE GROUPNAME RIGHTS {
		cui_action_area_new_for_acl_group_rights(window, background,
				$2.text, $3.text,
				fn, $3.begin, $3.end);
       }
    | MASK_RIGHTS_LINE RIGHTS {
		cui_action_area_new_for_acl_mask_rights(window, background,
				$2.text, fn, $2.begin, $2.end);
        }
    ;

%%

extern int getfacl_lineno;

int getfacl_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	
	error_message = g_strdup_printf(
		"Getfacl Parsing Error: %s in line %d", 
		c,
		getfacl_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return(1);
}

int getfacl_wrap(void)
{
#ifdef DEBUG
	g_message("%s(): returning 1", __func__);
#endif
	return 1;
}
