/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: icons.c,v 1.1 2006/02/24 17:19:21 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: icons.c,v $
 * Revision 1.1  2006/02/24 17:19:21  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#ifdef ALLDEBUG
#  define FILENAME_DEBUG
#endif


#include <stdlib.h>
#include <gnome.h>
//removed for porting
//#include <eel/eel.h>
#include "configure.h"
#include "icons.h"

/*****************************************************************************
 * Instantiable classed type: CuiIcon                                    *
 *                                                                           *
 *****************************************************************************/
static void
cui_icon_set_property(GObject *object, 
		guint           property_id, 
		const GValue    *value, 
		GParamSpec      *pspec);
static void
cui_icon_get_property(GObject *object, 
		guint            property_id, 
		GValue           *value, 
		GParamSpec       *pspec);

static void 
cui_icon_class_init(
		gpointer g_class, 
		gpointer g_class_data) {
	GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
	CuiIconClass *klass = CUI_ICON_CLASS(g_class);
	GParamSpec *pspec;
	
	gobject_class->set_property = cui_icon_set_property;
  	gobject_class->get_property = cui_icon_get_property;
}

static void
cui_icon_base_class_init(CuiIconClass *klass)
{
	
}

static void
cui_icon_instance_init(
		GTypeInstance   *instance, 
		gpointer         g_class)
{
	CuiIcon *self = (CuiIcon *)instance;
	CuiIconClass *klass = (CuiIconClass *)g_class;

	self->pixbuf = NULL;
}

static void
cui_icon_set_property(GObject *object, 
		guint           property_id, 
		const GValue    *value, 
		GParamSpec      *pspec)
{
	CuiIcon *self = (CuiIcon *)object;
}

static void
cui_icon_get_property(GObject *object, 
		guint            property_id, 
		GValue           *value, 
		GParamSpec       *pspec)
{
	CuiIcon *self = (CuiIcon *) object;
}

GType 
cui_icon_get_type(void) 
{
	static GType type = 0;
	
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof(CuiIconClass),
			(GBaseInitFunc) cui_icon_base_class_init, 
			(GBaseFinalizeFunc) NULL, 
			(GClassInitFunc) cui_icon_class_init, 
			NULL,
			NULL,
			sizeof(CuiIcon),
			0,
			(GInstanceInitFunc) cui_icon_instance_init 
		};
		type = g_type_register_static(G_TYPE_OBJECT, 
				"CuiIcon", &info, 0);
	}
	return type;
}

typedef struct iconfile {
	gchar *pattern;
	gchar *iconfilename;
} iconfile;

iconfile filetype_icons[] = {
	{"*.txt",          "txt.png"},
	{"*.text",         "txt.png"},
	{"*.tex",          "tex.png"},
	{"*.tar",          "tar.png"},
	{"*.xls",          "xls.png"},
	{"*.xml",          "source_xml.png"},
	{"*.py",           "source_python.png"},
	{"*.pl",           "source_pl.png"},
	{"*.php",          "source_php.png"},
	{"*.java",         "source_java.png"},
	{"*.h",            "source_h.png"},
	{"*.cpp",          "source_cpp.png"},
	{"*.c",            "source_c.png"},
	{"*.rpm",          "rpm.png"},
	{"*.ps",           "postscript.png"},
	{"*.eps",          "postscript.png"},
	{"*.odt",          "ooo_writer.png"},
	{"*.mid",          "midi.png"},
	{"*.[1-9]",        "man.png"},
	{"[Mm]akefile",    "make.png"},
	{"README",         "info.png"},
	{"*.[Aa]vi",       "avi.png"},
	{"*.[Mm]ov",       "mov.png"},
	{"*.[Mm][Pp][Gg]", "mpg.png"},
	{"*.[Mm]peg",      "mpg.png"},
	{"*.asf",          "asf.png"},
	{"*.wmf",          "wmf.png"},
	{"*.svg",          "svg.png"},
	{NULL, NULL}
};

static void
init_icon_by_file(CuiIcon *icon, CuiFilename *file)
{
	gchar *filename, *tmp;
	GdkPixbuf *pixbuf;
	
	filename = cui_filename_get_canonical_filename(file, "");
	/* 
	 * FIXME: this should be somewhere else...
	 * 
	 */
	if (filename[0] == '~'){
		tmp = g_build_filename(getenv("HOME"), filename + 2, NULL);
		g_free(filename);
		filename = tmp;
	}
	
	pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
	if (pixbuf != NULL)
		icon->pixbuf = pixbuf; 
		//removed for porting
		//icon->pixbuf = eel_gdk_pixbuf_scale_down_to_fit( pixbuf, 50, 50);
	g_free(filename);
}

static void
init_icon_by_filename(CuiIcon *icon, CuiFilename *file)
{
	gint n;
	gchar *filename;
	gchar *iconfilename = NULL;

	g_return_if_fail(CUI_IS_ICON(icon));
	g_return_if_fail(filename != NULL);

	filename = cui_filename_get_filename(file);
	
	n = 0;
	while (filetype_icons[n].pattern != NULL){
		if (fnmatch(filetype_icons[n].pattern, filename, 0) == 0){
			iconfilename = g_build_filename(PIXMAP_DIR, 
					filetype_icons[n].iconfilename, NULL);
			break;
		}
		++n;
	}

	if (iconfilename == NULL)
		iconfilename = g_build_filename(
				PIXMAP_DIR, "unknown.png", NULL);
		
	icon->pixbuf = gdk_pixbuf_new_from_file(iconfilename, NULL);
	if (icon->pixbuf == NULL)
		g_warning("%s(): Could not create icon.", __func__);
	
	g_free(filename);
}

/**
 * cui_icon_new:
 * @file: a #CuiFilename 
 * @returns: a new #CuiIcon object 
 *
 * Creates a new icon for the given file. FIXME: this function should be
 * completed.
 */
CuiIcon *
cui_icon_new(CuiFilename *file)
{
	CuiIcon *icon;
	
	g_return_val_if_fail(CUI_IS_FILENAME(file), NULL);
	icon = g_object_new(CUI_ICON_TYPE, NULL);
	
	init_icon_by_file(icon, file);
	if (icon->pixbuf == NULL)
		init_icon_by_filename(icon, file);
	return icon;
}

/**
 * cui_icon_get_pixbuf:
 *
 */
GdkPixbuf *
cui_icon_get_pixbuf(CuiIcon *icon)
{
	g_return_val_if_fail(CUI_IS_ICON(icon), NULL);
	return icon->pixbuf;
}

