%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: ls_lexer.l,v 1.1 2006/02/24 09:02:40 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: ls_lexer.l,v $
 * Revision 1.1  2006/02/24 09:02:40  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef ALLDEBUG
#  define LS_DEBUG
#endif

/*#define LS_DEBUG*/

#include <gnome.h>
#include <string.h>
#include "ls_parser_c.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser.h"

 /*#define LS_DEBUG */

#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%option yylineno

FS    [ \t]+
MONTH [[:upper:][:lower:]ÖÜÓŐÚÉÁŰöüóőúéáű][[:lower:]öüóőúéáű]*
YEAR  [0-9]{4,4}
DATE  [0-9]{1,2}
TIME  [0-9]{1,2}:[0-9]{1,2}
CYEAR [0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]

%s SHORT SHORTKNOWN LONG LONGLINK PRENAME LINKPRENAME NAME LINKNAME LINKORIGIN

%%
  cui_window_set_title(window_for_lex, _("Directory Content"));
  if (window_for_lex->parsed_chars == 0)
    if (parse_tree_check_option(terminal_for_lex->parse_tree, "ls", "l"))
      BEGIN(LONG);
    else
      if (parse_tree_check_option(terminal_for_lex->parse_tree, "ls", "p"))
        BEGIN(SHORTKNOWN);
      else
        BEGIN(SHORT);

<SHORT,SHORTKNOWN,LONG,LONGLINK>[ \t\n\r]+ {
	/* White chars: just for skipping to the next token */
	cui_window_get_token_marks(window_for_lex, 
		yytext, NULL, NULL);	
}

<SHORT,SHORTKNOWN,LONG,LONGLINK>^"ls: "[^\n\r]+ {
	/*
	 * These are error messages.
	 */
#ifdef LS_DEBUG
	g_message("%s: ERROR_MESSAGE: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
  	return(ERROR_MESSAGE);
}


<SHORT>(\"[^\"]+\")|('[^']+') {
	/* 
	 * Quoted string which may contain spaces. We do not know the type of
	 * this directory entry. 
	 */
#ifdef LS_DEBUG
	g_message("%s: STRING: '%s'", __func__, yytext);
#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	ls_lval.sval.text = strdup(yytext + 1);
	ls_lval.sval.text[strlen(ls_lval.sval.text) - 1] = '\0';
  	return(STRING);
}


<SHORTKNOWN>(\"[^\"]+\")|('[^']+') {
	/* 
	 * Quoted string which may contain spaces. We do know in this state that
	 * this is a regular file.
	 */
#ifdef LS_DEBUG
	g_message("%s: FILENAME: '%s'", __func__, yytext);
#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	ls_lval.sval.text = strdup(yytext + 1);
	ls_lval.sval.text[strlen(ls_lval.sval.text) - 1] = '\0';
  	return(FILENAME);
}


<SHORT,SHORTKNOWN>((\"[^\"]+\")|('[^']+'))\/ {
	/* 
	 * Quoted string with a slash after it. This is a directory.
	 */
#ifdef LS_DEBUG
	g_message("%s: DIRNAME: '%s'", __func__, yytext);
#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	ls_lval.sval.text = strdup(yytext + 1);
	ls_lval.sval.text[strlen(ls_lval.sval.text) - 2] = '\0';
  	return(DIRNAME);
}


<SHORTKNOWN>((\"[^\"]+\")|('[^']+'))@ {
	/* 
	 * Quoted string with an @ after it. This is a soft link.
	 */
#ifdef LS_DEBUG
	g_message("%s: LINKNAME: '%s'", __func__, yytext);
#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	ls_lval.sval.text = strdup(yytext + 1);
	ls_lval.sval.text[strlen(ls_lval.sval.text) - 2] = '\0';
  	return(LINKNAME);
}


<SHORT,SHORTKNOWN>[^\t \n\r]+\/ {
#ifdef LS_DEBUG
	g_message("%s: DIRNAME: '%s'", __func__, yytext);
#endif
	/* A directory name */
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	ls_lval.sval.text[strlen(ls_lval.sval.text) - 1] = '\0';
  	return(DIRNAME);
}


<SHORT,SHORTKNOWN>[^\t \n\r]+@ {
	/* 
	 * A soft link's name.
	 */
#ifdef LS_DEBUG
	g_message("%s: LINKNAME: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	ls_lval.sval.text[strlen(ls_lval.sval.text) - 1] = '\0';
  	return(LINKNAME);
}


<SHORT>[^\t \n\r]+ {
	/* Simple string, perhaps a regular file name */
#ifdef LS_DEBUG
	g_message("%s: STRING: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
  	return(STRING);
}


<SHORTKNOWN>[^\t \n\r]+ {
	/* 
	 * Simple string is a regular file name in this state.
	 */
#ifdef LS_DEBUG
	g_message("%s: FILENAME: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
  	return(FILENAME);
}


<LONG>^[^ \t]+[ \t]+[0-9,\.]+[KMG]?$ {
	/*
	 * The grand total. We don't know the exact text since it can be changed
	 * by the language settings.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_TOTAL: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		ls_lval.sval.begin, 
		ls_lval.sval.end); 
	return L_TOTAL;
}


<LONG>^[-dpcbs]([-r][-w][-xst]){3,3}[+]? {
	/*
         * The first character is the type of the directory entry.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_MODE: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_MODE;
}


<LONG>^"l"([-r][-w][-xst]){3,3} {
	/*
         * The first character is the type of the directory entry.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_MODE: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
		BEGIN(LONGLINK);
	return L_MODE;
}


<LONG,LONGLINK>[0-9]+[\.,]?[0-9]* {
	/*
 	 * Numbers for file links and filesizes.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_NUMBER: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	ls_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_NUMBER;
}

<LONG,LONGLINK>[0-9]+[\.,]?[0-9]*[kK] {
	/*
 	 * Numbers for file links and filesizes.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_NUMBER: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	ls_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	ls_lval.sval.double_val *= 1024;
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_NUMBER;
}

<LONG,LONGLINK>[0-9]+[\.,]?[0-9]*[mM] {
	/*
 	 * Numbers for file links and filesizes.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_NUMBER: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	ls_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	ls_lval.sval.double_val *= 1024 * 1024;
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_NUMBER;
}


<LONG,LONGLINK>[0-9]+[\.,]?[0-9]*[gG] {
	/*
 	 * Numbers for file links and filesizes.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_NUMBER: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	ls_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	ls_lval.sval.double_val *= 1024 * 1024 * 1024;
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_NUMBER;
}


<LONG>[0-9]+","[ \t\n\r]+[0-9]+ {
	/*
	 * Device node numbers.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_DNUMBERS: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex,
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_DNUMBERS;
}


<LONG>({MONTH}{FS}{DATE}{FS}({TIME}|{YEAR}))|({CYEAR}{FS}{TIME}) {
	/*
	 * Timestamps for modification dates.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_TIMESTAMP: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	BEGIN(PRENAME);
	return L_TIMESTAMP;
}


<LONGLINK>({MONTH}{FS}{DATE}{FS}({TIME}|{YEAR}))|({CYEAR}{FS}{TIME}) {
	/*
	 * Timestamps for modification dates.
	 */
#ifdef LS_DEBUG
	g_message("%s: L_TIMESTAMP: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	BEGIN(LINKPRENAME);
	return L_TIMESTAMP;
}


<LINKNAME>" -> " {
	/*
	 * FIXME: How to handle soft links?
	 */
#ifdef LS_DEBUG
	g_message("%s: L_ARROW: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	BEGIN(LINKORIGIN);
	return L_ARROW;
}


<LONG,LONGLINK>[a-zA-Z0-9\+-\.\/_]+ {
#ifdef LS_DEBUG
	g_message("%s: L_STRING: '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_STRING;
}


<PRENAME>[ \t\n\r]+ {
	/* 
	 * White chars after the timestamp and before the name: just skipping
	 * but the next state will accept the filename.
	 */
#ifdef LS_DEBUG
	g_message("%s: PRENAME: '%s'", __func__, yytext);
#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(NAME);
}


<LINKPRENAME>[ \t\n\r]+ {
	/* 
	 * White chars after the timestamp and before the link name: just skipping
	 * but the next state will accept the filename.
	 */
#ifdef LS_DEBUG
	g_message("%s: PRENAME: '%s'", __func__, yytext);
#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(LINKNAME);
}


<NAME>[^\n\r]+ {
#ifdef LS_DEBUG
	g_message("%s: L_STRING(NAME): '%s'", __func__, yytext);
#endif
	/* 
	 * Every string in this state is a name 
	 */
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	BEGIN(LONG);
  	return(L_STRING);
}


<LINKNAME>
[a-zA-Z0-9\+-\.\/_]+([ ]+[a-zA-Z0-9\+\.\/_]|[a-zA-Z0-9\+-\.\/_]{2,})* {
#ifdef LS_DEBUG
	g_message("%s: L_STRING(LINKNAME): '%s'", __func__, yytext);
#endif
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	return L_STRING;
}


<LINKORIGIN>[^\n\r]+ {
#ifdef LS_DEBUG
	g_message("%s: L_STRING(LINKORIGIN): '%s'", __func__, yytext);
#endif
	/* 
	 * Every string in this state is a name 
	 */
	ls_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ls_lval.sval.text,
		&ls_lval.sval.begin,
		&ls_lval.sval.end);
	BEGIN(LONG);
  	return(L_STRING);
}


<<EOF>> {
	return END_OF_WINDOW;
}

%%



