%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: make_parser.y,v 1.1 2006/02/24 08:07:47 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: make_parser.y,v $
 * Revision 1.1  2006/02/24 08:07:47  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "make_parser.h"

extern CuiWindow *window_for_lex;
gchar *working_directory;

CuiWidget *
cui_action_area_new_for_filename_line_number(
		CuiWindow   *window,
		Background  *background,
		gchar       *directory,
		gchar       *filename,
		gchar       *linenumber,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	CuiFilename *fn;
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	CuiFileType filetype;
	GtkTextIter biter, eiter;
	gchar *tag = NULL;
	gchar *active_tag = NULL;
	
	g_assert(background != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	
	fn = cui_filename_new(filename, directory);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);
	
#ifdef DEBUG
	g_message("%s(): Creating action area for '%s'", __func__, filename);
#endif
	tooltip = g_strdup_printf(
			_("Double click to open '%s' at %s."), 
			filename, linenumber);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_append_file_command_with_arg(widget, 
			FileOpenWithLine, fn, linenumber);	
	//if (!cui_filename_is_dir(file))
	//	cui_widget_append_window_command(widget, WindowRefresh, window);
	return widget;
}

%}

%parse-param {CuiWindow *window}
%union { 
	CuiLexElement sval;
}
 
%token <sval> CFILE STRING COMMAND MAKE_COMMAND
%token <sval> NUMBER
%token <sval> ERROR WARNING
%token <sval> MESSAGE_POINT_INT MESSAGE_UNDECLARED
%token <sval> ENTERING LEAVING MAKE_FILENAME
%token <sval> NOTHING_TO_DO
%token        DELIM_START DELIM_END
%token <sval> ROOT_DIR DIRNAME DIRSEP

%type  <sval> directory

%%
lines
  : lines line
  | line
  ;

line
  : CFILE STRING
  | CFILE NUMBER ERROR error_message {
		Background *background;
		background = cui_widget_get_background(CUI_WIDGET(window));
		cui_action_area_new_for_filename_line_number(
			window, background, 
			working_directory, $1.text, $2.text,
			$1.begin, $2.end);
	}
  | CFILE NUMBER WARNING error_message {
		Background *background;
		background = cui_widget_get_background(CUI_WIDGET(window));
		cui_action_area_new_for_filename_line_number(
			window, background, 
			working_directory, $1.text, $2.text,
			$1.begin, $2.end);
	}
  | STRING
  | COMMAND string
  | MAKE_COMMAND string
  | MAKE_COMMAND NOTHING_TO_DO
  | MAKE_COMMAND ENTERING DELIM_START directory DELIM_END {
		g_free(working_directory);
		working_directory = g_strdup($4.text);
	}
  | MAKE_COMMAND LEAVING DELIM_START directory DELIM_END
  | error { window->error_code++; }
  ;

error_message
  : string
  | MESSAGE_POINT_INT
  | MESSAGE_UNDECLARED
  ;

string
  : STRING
  | string STRING
  | string NUMBER
  ;

directory
  : ROOT_DIR { 
		cui_widgets_new_for_directory(window, &$1);     
	}
  | directory DIRNAME { 
		$$ = cui_lex_element_add(&$1, &$2);
		cui_widgets_new_for_directory(window, &$$);     
	}
  | directory DIRSEP DIRNAME { 
		$$ = cui_lex_element_build_path(&$1, &$2, &$3); 
		cui_widgets_new_for_directory(window, &$$);   
	}
  ;
%%

extern int make_lineno;

int make_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	
	error_message = g_strdup_printf(
		"Make Error: %s in line %d", 
		c,
		make_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return 1;
}

int make_wrap(void)
{
	g_message("%s(): returning 1", __func__);
	return 1;
}
