%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: man_lexer.l,v 1.1 2006/02/24 08:58:09 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: man_lexer.l,v $
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "man_parser.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"

#define DEBUG

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))

%}

%option yylineno
%s      ST_HEADER ST_NORMAL ST_NAME ST_DESCRIPTION

FS       [ \t]+
LINK     [A-Z]+\([0-9]+\)
%%
	if (window_for_lex->parsed_chars == 0){
		cui_window_set_title(window_for_lex, _("Manual Page"));
		BEGIN(ST_HEADER);
	}

<ST_HEADER>^[^\n]*$ {
	#ifdef DEBUG
	g_message("%s(): HEADER: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_NORMAL);
	return HEADER;
}

^"NAME"$ {
	#ifdef DEBUG
	g_message("%s(): CHAPTER(name): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_NAME);
	return CHAPTER;
}

<ST_NAME>[a-z]+ {
	#ifdef DEBUG
	g_message("%s(): COMMAND: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "command_name", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_DESCRIPTION);
	return COMMAND;
}

<ST_DESCRIPTION>"-" {
	#ifdef DEBUG
	g_message("%s(): other char: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	return '-';
}

<ST_DESCRIPTION>[a-z][^\n]* {
	#ifdef DEBUG
	g_message("%s(): DESCRIPTION: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "comment", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_NORMAL);
	return DESCRIPTION;
}

^[A-Z ]+$ {
	#ifdef DEBUG
	g_message("%s(): CHAPTER: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return CHAPTER;
}

^[\t ]*--[^ \n\t]+ {
	#ifdef DEBUG
	g_message("%s(): LONGOPT: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return LONGOPT;
}


--[^ \n\t]+ {
	#ifdef DEBUG
	g_message("%s(): LONGOPT: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return LONGOPT;
}

-[^ \n\t]+ {
	#ifdef DEBUG
	g_message("%s(): OPT: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return OPT;
}

[^ \n\t]+ {
	#ifdef DEBUG
	g_message("%s(): WORD: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "normal", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return WORD;
}

{FS} {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

\n+ {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

%%
