%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: man_parser.y,v 1.1 2006/02/24 08:15:09 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: man_parser.y,v $
 * Revision 1.1  2006/02/24 08:15:09  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "bash_parser_c.h"
#include "cuilexelements.h"
#include "man_parser.h"
#include "terminal.h"
#include "background.h"
#include "cuiwindow.h"
#include "history.h"

extern CuiWindow *window_for_lex;

%}

%debug
%parse-param {CuiWindow *window}
%union { 
	CuiLexElement sval;
}

%token <sval> HEADER
%token <sval> CHAPTER
%token <sval> COMMAND DESCRIPTION
%token <sval> LONGOPT OPT WORD

%%
lines
    : lines line
    | line
    ;

line: HEADER
    | COMMAND '-' DESCRIPTION {
		program_add_info($1.text, $3.text);
	}
    | CHAPTER
    | textline
    | error { window->error_code++; }
    ;

textline
    : WORD
    | option
    | textline WORD
    | textline option
    ;

option
    : LONGOPT {
		gchar *command;
		command = parse_tree_get_first_argument(window->parse_tree);
		program_add_longopt(command, $1.text);
	}
    | OPT
    ;
%%

extern int man_lineno;

int man_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	error_message = g_strdup_printf("Error in File: %s in lineno %d",
			c,
			man_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return(1);
}

int man_wrap(void)
{
	g_message("%s(): returning 1", __func__);
	return 1;
}
