%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: ps_parser.y,v 1.1 2006/02/24 08:08:42 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: ps_parser.y,v $
 * Revision 1.1  2006/02/24 08:08:42  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "widgets.h"
#include "cuiwindow.h"

int ps_wrap(void);
int ps_error(CuiWindow *window, char *);

extern CuiWindow *window_for_lex;
static CuiLexElement fll;
static CuiFilename *fn;
%}

/*%debug*/
%parse-param {CuiWindow *window}
%union {
	CuiLexElement sval;
};

%token <sval> HEADER
%token <sval> USERNAME NUMBER FLOAT TERMINAL STAT TIME 
%token <sval> KERNELPR INITPR PROGRAM


%token <sval> BLOCK_DEVICE MOUNT_POINT PERCENT
%%

lines: lines line
     | line
     ;

line: HEADER
    | USERNAME NUMBER FLOAT FLOAT NUMBER NUMBER terminal STAT TIME TIME program {
		Background *background;
		CuiWidget *widget;
	
		/*
		 * User menu.
		 */
		widget = cui_user_menu_new($1.text);
		background = cui_widget_get_background(CUI_WIDGET(window));
	   	cui_widget_set_background(widget, background);
	   	cui_widget_set_area_by_marks(widget, $1.begin, $1.end);

		background = cui_widget_get_background(CUI_WIDGET(window));
		/*
		 * FIXME: From the man page of ps:
		 *
		 * "This column CPU usage is currently expressed as the 
		 * percentage of time spent running during the entire lifetime 
		 * of a process. This is not ideal, and it does not conform to 
		 * the standards that ps otherwise conforms to. CPU usage is 
		 * unlikely to add up to exactly 100%."
		 */
		widget = cui_rectangle_emboss_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget, $1.begin, $1.end);
		$3.double_val /= 100.0;
		if ($3.double_val < 0.7)
			cui_rectangle_set_color(CUI_RECTANGLE(widget), 
				"#87f387",
				"#44f344",
				"#bef3be"); 
		else
			cui_rectangle_set_color(CUI_RECTANGLE(widget), 
				"#f3acac",
				"#f37777",
				"#f3cbcb"); 
			cui_widget_set_width_by_percent(widget, $3.double_val);
	}
    | error { window->error_code++; }
    ;

terminal: '?'
	| TERMINAL
	;

program: INITPR
       | KERNELPR
       | PROGRAM
       ;

%%

extern int ps_lineno;

int ps_error(CuiWindow *window, char *c)
{
	gchar *error_message;
	
	error_message = g_strdup_printf(
		"Process Status Error: %s in line %d", 
		c, 
		ps_lineno);
	
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return 1;
}

int ps_wrap(void)
{
	return 1;
}

