%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: simple_parser.y,v 1.1 2006/02/24 08:22:19 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: simple_parser.y,v $
 * Revision 1.1  2006/02/24 08:22:19  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "ls_parser_c.h"
#include "simple_parser.h"
#include "complex_widgets.h"


extern CuiWindow *window_for_lex;

%}

%parse-param {CuiWindow *window}
%union { 
	CuiLexElement sval;
}


%token <sval> USERNAME
%token <sval> ERROR_MESSAGE
%token <sval> ROOT_DIR DIRNAME DIRSEP
%type  <sval> directory
%token <sval> FILENAME
%%
lines: lines line
     | line
     ;

line: whoami
    | pwd
    | locate
    | ERROR_MESSAGE  { g_warning("%s(): Unimplemented.", __func__);}
    | error          { window->error_code++; }
    |
    ;

whoami
    : USERNAME       { cui_widgets_new_for_filename(window, &$1);   }
    ;

pwd
    : directory
    ;

locate
    : directory                   { cui_widgets_new_for_directory(window, &$1);     }
    | directory FILENAME          { cui_widgets_new_for_fullpath(window, &$2, &$1); }
    | directory DIRSEP FILENAME   { cui_widgets_new_for_fullpath(window, &$3, &$1); }

    ;

directory
    : ROOT_DIR                    { cui_widgets_new_for_directory(window, &$1);     }
    | directory DIRNAME           { $$ = cui_lex_element_add(&$1, &$2);
                                    cui_widgets_new_for_directory(window, &$$);     }
    | directory DIRSEP DIRNAME    { $$ = cui_lex_element_build_path(&$1, &$2, &$3); 
                                    cui_widgets_new_for_directory(window, &$$);     }
    ;

%%

extern int simple_lineno;

int simple_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	
	error_message = g_strdup_printf(
		"Simple Parser Error: %s in line %d", 
		c, simple_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return 1;
}

int simple_wrap(void)
{
#ifdef DEBUG
	g_message("%s(): returning 1", __func__);
#endif
	return 1;
}
