%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: stat_parser.y,v 1.1 2006/02/24 08:10:26 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: stat_parser.y,v $
 * Revision 1.1  2006/02/24 08:10:26  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "widgets.h"
#include "cuiwindow.h"

int stat_wrap(void);
int stat_error(CuiWindow *window, char *);

extern CuiWindow *window_for_lex;
static CuiLexElement fll;
static CuiFilename *fn;
%}

/*%debug*/
%parse-param {CuiWindow *window}
%union {
	int ival;
	char cval;
	CuiLexElement sval;
};

%type  <sval> device access 
%type  <sval> user group 
%token <sval> STAT_ERROR_MESSAGE
%token <sval> FILENAME NUMBER
%token <sval> ID DATE WORD DEVICE ZONE TIME MODE_STRING
%token <sval> FILE_LINE SIZE_LINE DEVICE_LINE ACCESS_LINE MODIFY_LINE
%token <sval> CHANGE_LINE UID GID
%token <sval> BLOCKS IO_BLOCK INODE LINKS
%%

lines: lines line
     | line
     ;

line: STAT_ERROR_MESSAGE {
    		ls_place_error_message(window, $1.text, $1.begin, $1.end); 
		window->error_code++;
    	}
    | FILE_LINE '`'  FILENAME '\''  {
    		fll = $3;
		}
    | SIZE_LINE NUMBER BLOCKS NUMBER IO_BLOCK NUMBER typename 
    | DEVICE_LINE device INODE NUMBER LINKS NUMBER 
    | ACCESS_LINE access UID user GID group {
    		Background *background;
		CuiWidget *widget;
		background = cui_widget_get_background(CUI_WIDGET(window));
    	   	fn = cui_filename_new(fll.text, window->wd->str); 
           	cui_filename_set_mode_string(fn, $2.text);
	  	cui_filename_set_owner_and_group(fn, $4.text, $6.text);
   		
		/* Filename action area */
		cui_action_area_new_for_filename(window,
				background, fn,
				fll.begin, fll.end);
	   	cui_file_menu_new_for_filename(window, background, fn,
	   			fll.begin, fll.end);
		/* Mode action area */
	   	cui_action_area_new_for_mode(window, background, 
				fn, $2.begin, $2.end);
		cui_syntax_highlight_apply_filename(background,
	   			fn, fll.begin, fll.end);
		/* Mode menu */
		widget = cui_mode_menu_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget, $2.begin, $2.end);
		cui_mode_menu_set_filename(CUI_MODE_MENU(widget), fn);
		/* The Icon */	
		widget = cui_pixmap_new_thumbnail(fn);
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget, $5.begin, $5.end);
		cui_widget_move(widget, 0, 28);
		
		/* Filename entry */
		widget = cui_entry_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget,fll.begin,fll.end);
		cui_entry_set_default_text(CUI_ENTRY(widget), fll.text);
		cui_widget_append_file_command(widget, FileRename, fn);
		
		/*
		 * The owner...
		 */
		/* */
		widget = cui_action_area_new_for_owner(
			window,
			background,
			fn,
			$4.begin, $4.end);
		 
		/* Owner entry */
		widget = cui_entry_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget, $4.begin, $4.end);
		cui_entry_set_default_text(CUI_ENTRY(widget), $4.text);
		cui_widget_append_file_command(widget, FileChangeOwner, fn);

		/* User menu on the owner */
	   	widget = cui_user_menu_new($4.text);
	   	cui_widget_set_background(widget, background);
	   	cui_widget_set_area_by_marks(widget, $4.begin, $4.end);
		
		/*
		 * The group owner.
		 */
		cui_action_area_new_for_group(window, background, fn,
			$6.begin, $6.end);
		/* Group owner entry */
		widget = cui_entry_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_area_by_marks(widget, $6.begin, $6.end);
		cui_entry_set_default_text(CUI_ENTRY(widget), $6.text);
		cui_widget_append_file_command(widget, FileChangeGroup, fn);
		}
    | ACCESS_LINE DATE TIME ZONE {
    		Background *background;
		background = cui_widget_get_background(CUI_WIDGET(window));
	   	CuiWidget *cl;
		cl = cui_calendar_new();
		cui_widget_set_background(cl, background);
		cui_widget_set_area_by_marks(cl, $2.begin, $2.end);
		cui_calendar_set_default_date(CUI_CALENDAR(cl), $2.text);
		cui_widget_append_file_command(cl, FileChangeAccessDate, fn);
		}
    | MODIFY_LINE DATE TIME ZONE {
    		Background *background;
		background = cui_widget_get_background(CUI_WIDGET(window));
	   	CuiWidget *cl;
		cl = cui_calendar_new();
		cui_widget_set_background(cl, background);
		cui_widget_set_area_by_marks(cl, $2.begin, $2.end);
		cui_calendar_set_default_date(CUI_CALENDAR(cl), $2.text);
		cui_widget_append_file_command(cl, FileChangeModifyDate, fn);
		}
    | CHANGE_LINE DATE TIME ZONE {
	   	CuiCalendar *cl;
		//cl = cui_calendar_new();
		//cui_widget_set_background(CUI_WIDGET(cl), background);
		//cui_widget_set_area_by_marks(CUI_WIDGET(cl), $2.begin, $2.end);
		//cui_calendar_set_default_date(cl, $2.text);
		//cui_widget_append_file_command(CUI_WIDGET(cl), 
		//		FileChangeChangeDate, fn);
		}
    | error { window->error_code++; }
    ;

device: NUMBER '/' NUMBER
      ;

typename: WORD
	| typename WORD
	;
	
access: '(' NUMBER '/' MODE_STRING ')' { $$ = $4; }
      ;

user: '(' NUMBER '/' WORD ')' { $$ = $4; }
    ;

group: '(' NUMBER '/' WORD ')' { $$ = $4; }
     ;

%%

extern int stat_lineno;

int stat_error(CuiWindow *window, char *c)
{
	gchar *error_message;
	
	error_message = g_strdup_printf(
		"File Status Error: %s in line %d",
		c,
		stat_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return 1;
}

int stat_wrap(void)
{
	return 1;
}

