/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: terminal.c,v 1.1 2006/02/24 17:15:21 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: terminal.c,v $
 * Revision 1.1  2006/02/24 17:15:21  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include "terminal.h"
#include "terminal_parser.h"
#include "cuiterm.h"
#include "cuiwindow.h"
#include "configure.h"
#include "support.h"
#include "interface.h"
#include "widgets.h"
#include "background.h"
#include "bash_parser_c.h"
#include "dnd.h"

#if DEBUG_ALL
#  define DEBUG_TERM
#endif

//#define DEBUG

/*
 * Static declarations.
 */
static gboolean send_command_timeout_function(gpointer data);
static gboolean executed_timeout_function(gpointer data);
static gchar *get_style_name(gint code);
static gint insert_to_terminal(Terminal *t, gchar *line);

/*
 * This function should get the window size.
 */
void terminal_redraw_now(Terminal *t)
{
	GdkRegion *region;
	region = gdk_drawable_get_visible_region(
			GDK_DRAWABLE(t->background->window));
	gdk_window_invalidate_region(t->background->window, region, TRUE);
	/*
	 * I think this caused a flicker on the terminal widget.
	gdk_window_process_all_updates(); */
	gdk_region_destroy(region);
}

void
terminal_debug_print_status(Terminal *t)
{
	switch (t->status){
		case TermUninitialized:
			g_message("%s(): status: TermUninitialized", __func__);
			break;
			
		case TermReady:
			g_message("%s(): status: TermReady", __func__);
			break;
			
		case TermSendingCommand:
			g_message("%s(): status: TermSendingCommand", __func__);
			break;
			
		case TermSendingUserEnter:
			g_message("%s(): status: TermSendingUserEnter", 
					__func__);
			break;
			
		case TermExecuting:
			g_message("%s(): status: TermExecuting", __func__);
			break;
			
		default:
			g_message("%s(): status: Unknown: %d", __func__,
					t->status);
	}
}

/**************************************************************************
 * Terminal control sequence handling functions.                          *
 *                                                                        *
 **************************************************************************/
gint 
terminal_sequence_overwrite(Terminal *t) 
{
#ifdef DEBUG
	g_message("%s(): Setting to overwrite.", __func__);
#endif
	t->overwrite = TRUE;
	return 0;
}

gint 
terminal_sequence_insert(Terminal *t) 
{
#ifdef DEBUG
	g_message("%s(): Setting to insert.", __func__);
#endif
	t->overwrite = FALSE;
	return 0;
}

/*
 * The terminal gets some text which overwrites the actual screen if we are not
 * in insert mode.
 */
gint 
terminal_sequence_text_with_attributes(Terminal *t, 
		gchar *buffer, 
		gint *args, 
		gint argn)
{
	gint n;
	gchar *style;
	GtkTextMark *cursor, *oldbegin;
	GtkTextIter begin, end;
	gint length = g_utf8_strlen(buffer, -1);
	
	if (length == 0)
		return 0;
#ifdef DEBUG
	g_message("%s(): Printing '%s' length %d.", __func__, 
			buffer, length);
#endif
	if (t->overwrite)
		terminal_sequence_delete_n(t, length);
	/*
	 * We insert the text with the first style.
	 */
	style = "normal";
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	oldbegin = gtk_text_buffer_create_mark(t->text_buffer,
                                             "oldbegin",
                                             &end, TRUE);
	gtk_text_buffer_insert_with_tags_by_name(t->text_buffer, &end, 
			buffer, -1, style, NULL);
	/*
	 * Then we apply the followin styles.
	 */
	oldbegin = gtk_text_buffer_get_mark(t->text_buffer, "oldbegin");
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &begin, oldbegin);
	for (n = 0; n < argn; ++n){
		//printf("arg[%d] = %d   ", n, args[n]);
		style = get_style_name(args[n]);
		gtk_text_buffer_apply_tag_by_name(t->text_buffer,
                                             style, &begin, &end);
	}
	//printf("\n");
	//fflush(stdout);
	/* We want to be sure that the cursor is visible, and the
	 * text_view has the focus */
	//cursor = gtk_text_buffer_get_insert(t->text_buffer);
	//gtk_text_view_scroll_mark_onscreen(t->text_view, cursor);
	gtk_widget_grab_focus(GTK_WIDGET(t->text_view));

	return n;
}

gint 
terminal_sequence_prompt(Terminal *t, gchar *buffer)
{
	GtkTextMark *cursor;
	GtkTextIter iter;
	gint length = g_utf8_strlen(buffer, -1);
	
	if (length == 0)
		return 0;
#ifdef DEBUG
	g_message("%s(): Printing '%s' length %d.", __func__, 
			buffer, length);
#endif
	if (t->overwrite)
		terminal_sequence_delete_n(t, length);
		
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_buffer_insert_with_tags_by_name(t->text_buffer, &iter, 
			buffer, -1, 
			"command_line", NULL);
	
	/* We want to be sure that the cursor is visible */
	//cursor = gtk_text_buffer_get_insert(t->text_buffer);
	//gtk_text_view_scroll_mark_onscreen(t->text_view, cursor);

	/* We have to get the keyboard focus is we want the user to type. */
	gtk_widget_grab_focus(GTK_WIDGET(t->text_view));
	return length;
}

gint 
terminal_sequence_text(Terminal *t, gchar *buffer)
{
	GtkTextMark *cursor;
	GtkTextIter iter;
	gint length = g_utf8_strlen(buffer, -1);
	
	if (length == 0)
		return 0;
#ifdef DEBUG
	g_message("%s(): Printing '%s' length %d.", __func__, 
			buffer, length);
#endif
	if (t->overwrite)
		terminal_sequence_delete_n(t, length);
		
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_buffer_insert_with_tags_by_name(t->text_buffer, &iter, 
			buffer, -1, 
			"normal", NULL);
	
	/* We want to be sure that the cursor is visible */
	//cursor = gtk_text_buffer_get_insert(t->text_buffer);
	//gtk_text_view_scroll_mark_onscreen(t->text_view, cursor);

	/* We have to get the keyboard focus is we want the user to type. */
	gtk_widget_grab_focus(GTK_WIDGET(t->text_view));
	return length;
}

static void
terminal_create_workscape_menu(Terminal *t)
{
	CuiWidget *workspace_menu;
	
	workspace_menu = cui_popup_menu_new(create_application_popup_menu());
	cui_widget_set_area_by_coord(workspace_menu, 0, 0, 780, 1);
	cui_widget_set_name(workspace_menu, "WorkspaceMenu");
	cui_widget_set_background(workspace_menu, t->background);
}

void
terminal_sequence_erase(Terminal *t)
{
	GtkTextIter start, end;
	/*
	 * Deleting all widgets.
	 */
	cui_background_erase(t->background);
	/*
	 * Deleting text.
	 */
	gtk_text_buffer_get_start_iter(t->text_buffer, &start);
	gtk_text_buffer_get_end_iter(t->text_buffer, &end);
	gtk_text_buffer_delete(t->text_buffer, &start, &end);
	/*
	 * A new workspace_menu.
	 */
	terminal_create_workscape_menu(t);
}

void 
terminal_sequence_backspace(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter begin, end;

#ifdef DEBUG
	g_message("%s(): Delete backward.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &begin, cursor);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	gtk_text_iter_backward_chars(&begin, 1);
	gtk_text_buffer_delete(t->text_buffer, &begin, &end);
}


void 
terminal_sequence_backspace_n(Terminal *t, gint n)
{
	GtkTextMark *cursor;
	GtkTextIter begin, end;

#ifdef DEBUG
	g_message("%s(): Delete backward %d", __func__, n);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &begin, cursor);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	gtk_text_iter_backward_chars(&begin, n);
	gtk_text_buffer_delete(t->text_buffer, &begin, &end);
}


void 
terminal_sequence_delete(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter begin, end;

#ifdef DEBUG
	g_message("%s(): Delete.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &begin, cursor);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	gtk_text_iter_forward_chars(&end, 1);
	gtk_text_buffer_delete(t->text_buffer, &begin, &end);
}


void 
terminal_sequence_delete_to_end(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter begin, end;
	gchar *tmp;
#ifdef DEBUG
	g_message("%s(): Delete to line end.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &begin, cursor);
	/*
	 * If the begin iterator is at the end of line, the forward_to_line_end
	 * brings us to the next line. However in that case we don't have to
	 * delete anything.
	 */
	if (gtk_text_iter_ends_line(&begin))
		return;
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	gtk_text_iter_forward_to_line_end(&end);

#ifdef DEBUG	
	tmp = gtk_text_buffer_get_text(t->text_buffer, &begin, &end, TRUE);
	g_message("%s(): deleting text: '%s'", __func__, tmp);
	g_free(tmp);
#endif
	
	gtk_text_buffer_delete(t->text_buffer, &begin, &end);
	
#ifdef DEBUG
	g_message("%s(): End", __func__);
#endif
}


void 
terminal_sequence_delete_to_begin(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter begin, end;
	gint line_num;

#ifdef DEBUG
	g_message("%s(): Delete to line begin.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	line_num = gtk_text_iter_get_line(&end);
	gtk_text_buffer_get_iter_at_line(t->text_buffer, &begin, line_num);
	gtk_text_buffer_delete(t->text_buffer, &begin, &end);
}


void 
terminal_sequence_delete_line(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter begin, end;
	gint line_num;
	
#ifdef DEBUG
	g_message("%s(): Delete to line begin.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	gtk_text_iter_forward_to_line_end(&end);
	line_num = gtk_text_iter_get_line(&end);
	gtk_text_buffer_get_iter_at_line(t->text_buffer, &begin, line_num);
	gtk_text_buffer_delete(t->text_buffer, &begin, &end);
}


void 
terminal_sequence_delete_n(Terminal *t, gint n)
{
	GtkTextMark *cursor;
	GtkTextIter begin, end;
	gchar *tmp;
#ifdef DEBUG
	g_message("%s(): Deleting %d characters.", __func__, n);
#endif
	if (n == 0)
		return;

	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &begin, cursor);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, cursor);
	gtk_text_iter_forward_chars(&end, n);
#if 0	
	if (gtk_text_iter_ends_line(&begin))
		g_message("begin at end of line");
	if (gtk_text_iter_ends_line(&end))
		g_message("end at end of line");
#endif

#ifdef DEBUG
	tmp = gtk_text_buffer_get_text(t->text_buffer, &begin, &end, TRUE);
	g_message("%s(): deleting text: '%s' %d", __func__, tmp, tmp[0]);
	g_free(tmp);
#endif
	
	gtk_text_buffer_delete(t->text_buffer, &begin, &end);
	return;
}

void
terminal_sequence_cursor_xy(Terminal *t, gint x, gint y)
{
	GdkRectangle visible_rect;
	GtkTextMark  *cursor;
	gint window_x, window_y;
	GtkTextIter iter;
	gint n;
#ifdef DEBUG
	g_message("%s(): Moving to <%d, %d>.", __func__, x, y);
#endif
	g_message("%s(): Moving to <%d, %d>.", __func__, x, y);
	gtk_text_view_get_visible_rect(t->text_view, &visible_rect);
	g_message("x: %d, y: %d, width: %d, height: %d",
			visible_rect.x,
			visible_rect.y,
			visible_rect.width,
			visible_rect.height
			);
	gtk_text_view_buffer_to_window_coords(t->text_view, 
			GTK_TEXT_WINDOW_TEXT, 
			visible_rect.x, visible_rect.y, 
			&window_x, &window_y);
	//window_x = visible_rect.x;
	//window_y = visible_rect.y;
	g_message("wx: %d, wy: %d",
			window_x,
			window_y);

	gtk_text_view_get_line_at_y(t->text_view, &iter, window_y, NULL);

	for (n = 1; n < y; ++n){
		//g_message("line n: %d", n);
		if (!gtk_text_iter_forward_line(&iter)){
			gtk_text_iter_forward_to_line_end(&iter);
			gtk_text_buffer_insert_with_tags_by_name(
					t->text_buffer, &iter, 
					"\r\n", -1, "normal", NULL);
			//g_message("inserting line");
			//gtk_text_iter_forward_line(&iter);
		}
		//g_message("forward line");
	}


	for (n = 0; n < x; ++n){
		//g_message("char n: %d", n);
		if (!gtk_text_iter_forward_char(&iter)){
			gtk_text_buffer_insert_with_tags_by_name(
					t->text_buffer, &iter, 
					" ", -1, "normal", NULL);
			//g_message("inserting char");
			gtk_text_iter_forward_char(&iter);
		}
		//g_message("forward char");
	}

	gtk_text_iter_backward_char(&iter);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
	//cursor = gtk_text_buffer_get_insert(t->text_buffer);
	//gtk_text_view_scroll_mark_onscreen(t->text_view, cursor);
	
}

void
terminal_sequence_cursor_home(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter iter;
	GtkTextIter end; //
	gchar *text;
	gint line_num;
#ifdef DEBUG
	g_message("%s(): Moving home.", __func__);	
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);

	gtk_text_buffer_get_end_iter(t->text_buffer, &end); //
	while (!gtk_text_iter_starts_line(&iter)){
		text = gtk_text_buffer_get_text(t->text_buffer, &iter, &end,
				TRUE); //
		if (!gtk_text_iter_backward_char(&iter))
			break;
	}
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}


void
terminal_sequence_cursor_up(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter iter;
#ifdef DEBUG
	g_message("%s(): Moving up.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_backward_line(&iter);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}

void
terminal_sequence_cursor_down(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter iter;
#ifdef DEBUG
	g_message("%s(): Moving down.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_forward_line(&iter);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}


void
terminal_sequence_cursor_down_n(Terminal *t, gint n)
{
	GtkTextMark *cursor;
	GtkTextIter iter;
#ifdef DEBUG
	g_message("%s(): Moving down %d", __func__, n);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_forward_lines(&iter, n);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}

void
terminal_sequence_cursor_right(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter iter;
#ifdef DEBUG
	g_message("%s(): Moving right.", __func__);
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_forward_char(&iter);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}

void
terminal_sequence_cursor_right_n(Terminal *t, gint n)
{
	GtkTextMark *cursor;
	GtkTextIter iter;

#ifdef DEBUG
	g_message("%s(): Moving right %d", __func__, n);	
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_forward_chars(&iter, n);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}

void
terminal_sequence_cursor_left(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter iter;

#ifdef DEBUG
	g_message("%s(): Moving left.", __func__);	
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_backward_char(&iter);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}


void
terminal_sequence_cursor_left_n(Terminal *t, gint n)
{
	GtkTextMark *cursor;
	GtkTextIter iter;

#ifdef DEBUG
	g_message("%s(): Moving left %d", __func__, n);	
#endif
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_backward_chars(&iter, n);
	gtk_text_buffer_place_cursor(t->text_buffer, &iter);
}

/***************************************************************************
 * Other static functions handling character terminal.                     *
 *                                                                         *
 **************************************************************************/

static gboolean
command_executed(gpointer data)
{
	CuiWindow   *window;
	Terminal    *t = (Terminal *)data;	
	GtkTextMark *mark;
	GtkTextMark *cursor;
	GtkTextIter iter;
#ifdef DEBUG_TERM
	g_message("%s(): start", __func__);
#endif
	window = CUI_WINDOW(t->last_command_window);
	if (window == NULL){
		g_warning("Embedded window is NULL.");
		return FALSE;
	}
	/*
	 * 
	 */
	/*
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, window->begin);
	gtk_text_iter_forward_line(&iter);
	gtk_text_iter_backward_char(&iter);
	gtk_text_buffer_move_mark(t->text_buffer, window->begin, &iter);
	*/
	/*
	 * When we are executed a program, we use the cursor position
	 * to calculate where is the last of the output lines. We store this
	 * position for the CuiWindow.
	 */
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	gtk_text_iter_backward_line(&iter);
	gtk_text_iter_forward_to_line_end(&iter);
	mark = gtk_text_buffer_create_mark(t->text_buffer, NULL, &iter, FALSE);
	
	cui_window_set_end_mark(window, mark);
	t->parsed_chars = 0;
	cui_window_set_parse_tree(window, t->parse_tree);
	cui_window_realize(window);
	terminal_redraw_now(t);
	/*
	 * Let's check if there are pending commands.
	 */
	if (g_list_length(t->pending_commands) != 0){
		terminal_send_command(t, t->pending_commands->data, TRUE);
		g_free(t->pending_commands->data);
		t->pending_commands = g_list_delete_link(
				t->pending_commands,
				t->pending_commands);
	}
#ifdef DEBUG_TERM
	g_message("%s(): end", __func__);
#endif
	return FALSE;
}

/*
 * This timeout function send the current command by writing a '\n' to the
 * terminal.
 */
static gboolean
send_command_timeout_function(gpointer data){
	gsize written;
	GError *error = NULL;
	Terminal *t = (Terminal *)data;	
	
	g_io_channel_write_chars(t->shell_stream,
			"\n", -1, &written, &error);
	terminal_command_starting(t);
	return FALSE;
}

extern Terminal *terminal_for_lex;

/*
 * Reads the typed in command line from the buffer and stores it. This function
 * assumes that the current line is built from a prompt, and the command line
 * itself.
 */
void
terminal_get_command(Terminal *t)
{
	GtkTextMark *cursor;
	GtkTextIter line_start, line_end;
	gint line_num;
	gchar *command = NULL;
	
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &line_start, cursor);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &line_end, cursor);
	
	line_num = gtk_text_iter_get_line(&line_start);
	gtk_text_iter_forward_to_line_end(&line_end);
	

	while(!gtk_text_iter_starts_line(&line_start))
		if (!gtk_text_iter_backward_char(&line_start))
			break;
	/*
	 *
	 */
	gtk_text_buffer_apply_tag_by_name(t->text_buffer, "command_line", 
			&line_start, &line_end);
	
	gtk_text_iter_forward_chars(&line_start, t->prompt_length);
	
	command = gtk_text_buffer_get_text(t->text_buffer, 
			&line_start, &line_end, TRUE);
	t->command = g_string_assign(t->command, command);
}

gboolean
type_timeout_function(gpointer data){
	Terminal *t = (Terminal *)data;	
	GtkTextIter line_start, line_end;
	gchar *command = NULL;
	gint line_num;
	GtkTextMark *cursor;

	g_assert(t != NULL);
	
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &line_start, cursor);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &line_end, cursor);
	
	/*
	 * The begin and end of the typed command left to the prompt.
	 */
	line_num = gtk_text_iter_get_line(&line_start);
	gtk_text_iter_forward_to_line_end(&line_end);
	while(!gtk_text_iter_starts_line(&line_start))
		if (!gtk_text_iter_backward_char(&line_start))
			break;
	gtk_text_iter_forward_chars(&line_start, t->prompt_length);

	/*
	gtk_text_buffer_remove_all_tags(t->text_buffer, 
			&line_start, &line_end);
	*/
	
	gtk_text_buffer_apply_tag_by_name(t->text_buffer, "normal", 
			&line_start, &line_end);
	
	terminal_for_lex = t;
	t->parser_cursor = gtk_text_buffer_create_mark(
			t->text_buffer, NULL,
                        &line_start, TRUE);	
	t->lexer_cursor = gtk_text_buffer_create_mark(
			t->text_buffer, NULL,
                        &line_start, TRUE);	
	/*
	 * We call the BASH parser which will make a syntax highlight on the
	 * command and create a parse tree. The parser returns 1 if there were 
	 * some error parsing the command. 
	 */
	t->parsed_chars = 0;
	t->parse_tree = NULL; /* FIXME: this should be freed. */
	bash_parse(t);
#if 0
	/* Some debug function to print the parse tree. */		
	if (t->parse_tree != NULL){
		printf("-----------Parse tree-----------\n");
		g_node_traverse(t->parse_tree, G_PRE_ORDER, 
			G_TRAVERSE_ALL, -1, 
			node_debug_print, NULL);
	}
#endif
	static guint timeout_id = 0;
	if (timeout_id != 0) {
		g_source_remove(timeout_id);
		timeout_id = 0;
	}
	/* FIXME: for this we can not destroy the parse tree? */
	timeout_id = g_timeout_add(500, 
			parse_tree_check, 
			(gpointer)(t->parse_tree));
finalize:
	g_free(command);
	return FALSE;
}


static gchar *
get_style_name(gint code)
{
	switch(code){
		case 1:
			return "Bright";
		case 2:
			return "Dim";
		case 3:
			return "Underscore";
		case 5:
			return "Blink";
		case 7:
			return "Reverse";
		case 8:
			return "Hidden";
		case 30:
			return "FgBlack";
		case 31:
			return "FgRed";
		case 32:
			return "FgGreen";
		case 33:
			return "FgYellow";
		case 34:
			return "FgBlue";
		case 35:
			return "FgMagenta";
		case 36:
			return "FgCyan";
		case 37:
			return "FgWhite";
		case 40:
			return "BgBlack";
		case 41:
			return "BgRed";
		case 42:
			return "BgGreen";
		case 43:
			return "BgYellow";
		case 44:
			return "BgBlue";
		case 45:
			return "BgMagenta";
		case 46:
			return "BgCyan";
		case 47:
			return "BgWhite";
		default:
			//g_warning("Unknown attribute code: %d", code);
			return "normal";
	}
}


static gchar *terminal_buffer = NULL;
static gchar *terminal_buffer_counter = NULL;

/*
 */
int
terminal_get_buffer_char(char *buf, int max_size)
{
	gint size;
#ifdef DEBUG_TERM
	g_message("%s(): Start", __func__);
#endif
	
	if (terminal_buffer == NULL || 
			terminal_buffer_counter == NULL ||
			*terminal_buffer_counter == '\0'){
#ifdef DEBUG_TERM
	g_message("%s(): returning 0", __func__);
#endif
		buf[0] = 0;
		return 0;
	}
	
	size = MIN(max_size, strlen(terminal_buffer_counter));
	strncpy(buf, terminal_buffer_counter, size);
	terminal_buffer_counter += size;	

#ifdef DEBUG_TERM
	g_message("%s(): got string: '%s'", __func__, buf);
#endif

	return size;
}

static gint 
insert_to_terminal(Terminal *t, gchar *line)
{
#ifdef DEBUG_TERM
	g_message("%s(): start", __func__);
#endif
	g_free(terminal_buffer);
	terminal_buffer = g_strdup(line);
	terminal_buffer_counter = terminal_buffer;
	terminal_parse(t);
#ifdef DEBUG_TERM
	g_message("%s(): end", __func__);
#endif
	return 0;
}


/***************************************************************************
 * Utility functions handling character terminal.                          *
 *                                                                         *
 **************************************************************************/
void on_main_widget_modified(GtkTextBuffer *textbuffer, 
		gpointer user_data)
{
	//g_warning("%s: called.", __func__);
}

static void
terminal_create_tags(Terminal *t)
{
	/* The normal text. */
	gtk_text_buffer_create_tag(t->text_buffer, "normal", 
				"family", "monospace", 
				NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "normal_active", 
			"family", "monospace", 
			"foreground", "DarkGray", NULL);
	/* 
	 * Terminal attributes.
	 */
	gtk_text_buffer_create_tag(t->text_buffer, "Bright", 
				"weight", PANGO_WEIGHT_BOLD, 
				NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "Dim", 
				"foreground", "#555555", NULL);
 	gtk_text_buffer_create_tag(t->text_buffer, "Underscore",
			      "underline", PANGO_UNDERLINE_SINGLE, NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "Blink",
				"weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "Reverse", 
				"foreground", "White", 
				"background", "Black", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "Hidden", 
				"foreground", "White", NULL);
	/* 
	 * Terminal foreground styles.
	 */
	gtk_text_buffer_create_tag(t->text_buffer, "FgBlack", 
				"foreground", "Black", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "FgRed", 
				"foreground", "Red", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "FgGreen", 
				"foreground", "Green", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "FgYellow", 
				"foreground", "Yellow", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "FgBlue", 
				"foreground", "Blue", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "FgMagenta", 
				"foreground", "Magenta", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "FgCyan", 
				"foreground", "Cyan", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "FgWhite", 
				"foreground", "White", NULL);
	/* 
	 * Terminal background styles. 
	 */
	gtk_text_buffer_create_tag(t->text_buffer, "BgBlack", 
				"background", "Black", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "BgRed", 
				"background", "Red", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "BgGreen", 
				"background", "Green", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "BgYellow", 
				"background", "Yellow", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "BgBlue", 
				"background", "Blue", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "BgMagenta", 
				"background", "Magenta", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "BgCyan", 
				"background", "Cyan", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "BgWhite", 
				"background", "White", NULL);
	/******************************************************************** 
	 * Syntax highlight styles for BASH parser.                         *
	 ********************************************************************/
	gtk_text_buffer_create_tag(t->text_buffer, "command_line", 
			"family", "monospace",
			"pixels_above_lines", 8,
			"pixels_below_lines", 8,
			NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "underline-error", 
			"underline", PANGO_UNDERLINE_ERROR, NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "command_name", 
			"family", "monospace",
			"foreground", "DarkGreen", 
			NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "argument", 
			"family", "monospace", 
			"foreground", "Black", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "option", 
			"family", "monospace", 
			"foreground", "Brown", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "keyword", 
			"family", "monospace", 
			"foreground", "Brown", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "list_element", 
			"family", "monospace", 
			"foreground", "Brown", NULL);

	gtk_text_buffer_create_tag(t->text_buffer, "variable", 
			"family", "monospace", 
			"foreground", "DarkGreen", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "embedded", 
			"family", "monospace", 
			"foreground", "purple", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "varvalue", 
			"family", "monospace", 
			"foreground", "purple", NULL);
	
	/*************************************************************
	 * Other styles, for various objects.                        *
	 *************************************************************/
	gtk_text_buffer_create_tag(t->text_buffer, "kernel_process", 
			"family", "monospace", 
			"foreground", "Red", NULL);

	gtk_text_buffer_create_tag(t->text_buffer, "comment", 
			"family", "monospace", 
			"style", PANGO_STYLE_ITALIC,
			"foreground", "DarkBlue", NULL);
	/* Directory entries */
	gtk_text_buffer_create_tag(t->text_buffer, "file", 
			"family", "monospace", 
			"foreground", "Black", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "file_active", 
			"family", "monospace", 
			"foreground", "DarkGray", NULL);

	gtk_text_buffer_create_tag(t->text_buffer, "directory", 
			"family", "monospace", 
			"foreground", "RoyalBlue4", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "directory_active", 
			"family", "monospace", 
			"foreground", "RoyalBlue1", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "executable", 
			"family", "monospace", 
			"foreground", "DarkGreen", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "executable_active", 
			"family", "monospace", 
			"foreground", "DarkSeaGreen", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "archive", 
			"family", "monospace", 
			"foreground", "IndianRed4", NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "archive_active", 
			"family", "monospace", 
			"foreground", "IndianRed1", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "block_device", 
			"family", "monospace", 
			"foreground", "RosyBrown4", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "block_device_active", 
			"family", "monospace", 
			"foreground", "RosyBrown1", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "character_device", 
			"family", "monospace", 
			"foreground", "RosyBrown4", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "character_device_active", 
			"family", "monospace", 
			"foreground", "RosyBrown1", NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "filename", 
			"family", "monospace", 
			"foreground", "brown",
			"style", PANGO_STYLE_ITALIC,
			NULL);
	/* user and group names */
	gtk_text_buffer_create_tag(t->text_buffer, "username", 
			"family", "monospace", 
			"foreground", "MediumPurple4",
			NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "username_active", 
			"family", "monospace", 
			"foreground", "MediumPurple1",
			NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "groupname", 
			"family", "monospace", 
			"foreground", "MediumPurple4",
			NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "groupname_active", 
			"family", "monospace", 
			"foreground", "MediumPurple1",
			NULL);
	/*
	 * Yes and no colors for user file rights and other areas.
	 */
	gtk_text_buffer_create_tag(t->text_buffer, "yes", 
			"family", "monospace", 
			"background", "SpringGreen4",
			NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "yes_active", 
			"family", "monospace", 
			"background", "SpringGreen1",
			NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "no", 
			"family", "monospace", 
			"background", "VioletRed4",
			NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "no_active", 
			"family", "monospace", 
			"background", "VioletRed1",
			NULL);

	gtk_text_buffer_create_tag(t->text_buffer, "unimportant", 
			"family", "monospace", 
			"foreground", "gray",
			"style", PANGO_STYLE_ITALIC,
			NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "important", 
			"family", "monospace", 
			"style", PANGO_STYLE_ITALIC,
			//"foreground", "Red", 
			NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "Header", 
			"family",     "monospace", 
			"style",      PANGO_STYLE_ITALIC,
			"variant",    PANGO_VARIANT_SMALL_CAPS,
			"weight",     PANGO_WEIGHT_HEAVY, 
			NULL);
	
	gtk_text_buffer_create_tag(t->text_buffer, "error_message", 
			"family", "monospace", 
			"foreground", "red",
			"style", PANGO_STYLE_ITALIC,
			NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "error", 
			"family", "monospace", 
			"foreground", "red",
			NULL);
	gtk_text_buffer_create_tag(t->text_buffer, "warning_message", 
			"family", "monospace", 
			"foreground", "yellow",
			//"style", PANGO_STYLE_ITALIC,
			NULL);
}


Terminal *
terminal_new(GtkTextView *text_view)
{
	GtkTextIter iter;
	CuiWidget *pixmap;
	Terminal *t = g_malloc(sizeof(Terminal));
	
	g_assert(text_view != NULL);
	g_assert(t != NULL);
	g_object_set_data(G_OBJECT(text_view), "terminal", t);

	t->status = TermUninitialized;
	t->shell_stream = NULL;
	t->last_command_window = NULL;
	
	t->text_view = text_view;
	t->text_buffer = gtk_text_view_get_buffer(text_view);

	g_signal_connect((gpointer) t->text_buffer, "end-user-action", 
			G_CALLBACK(on_main_widget_modified), NULL);

	t->hold_shift = 0;
	t->overwrite = TRUE;
	t->command = g_string_new("");
	t->parse_tree = NULL;
	t->cmd = g_string_new("");
	t->wd = g_string_new("");
	t->username = g_string_new("");
	t->hostname = g_string_new("");
	t->directory = g_string_new("");
	t->background = background_new(text_view);
	t->command_generator = cui_generator_new();
	cui_generator_set_terminal(t->command_generator, (const gpointer)t);

	/*
	 * Field holding information on pending commands;
	 */ 
	t->pending_commands = NULL;
	/*
	 * Creating tags for syntax highlight, terminal color attributes and so
	 * on.
	 */
	terminal_create_tags(t);
	/*
	 * The workspace menu.
	 */
	terminal_create_workscape_menu(t);
	/*
	 * Initializing the Drag & Drop facility.
	 */
	dnd_init_destination(GTK_WIDGET(text_view));

#if SPLASH
	/*
	 * The splash screen
	 */
	if (!bg_conf_get_bool("/init/splash_screen", TRUE))
		return t;
	
	gtk_text_buffer_get_start_iter(t->text_buffer, &iter);
	gtk_text_buffer_insert_with_tags_by_name(t->text_buffer, &iter, 
	_(
	"Composite User Interface Terminal (" PACKAGE " version " VERSION ")\n"
	"Copyright (c) 2005, 2006 Pere Laszlo\n"
	"This is free software; see the source for copying conditions.\n"
	"\n"
	), -1, "Dim", NULL);

	if (bg_conf_get_bool("/init/distr_logo", TRUE)){
		if (g_file_test(FEDORA_RELEASE, G_FILE_TEST_EXISTS)){
	  		pixmap = cui_pixmap_new_from_package_file("fedora.png");
			cui_pixmap_set_xy(CUI_PIXMAP(pixmap), 670, 5);
		}else if (g_file_test(GENTOO_RELEASE, G_FILE_TEST_EXISTS)){
			pixmap = cui_pixmap_new_from_package_file("gentoo.png");
			cui_pixmap_set_xy(CUI_PIXMAP(pixmap), 750, 10);
		}else if (g_file_test(DEBIAN_RELEASE, G_FILE_TEST_EXISTS)){
	  		pixmap = cui_pixmap_new_from_package_file("debian.png");
			cui_pixmap_set_xy(CUI_PIXMAP(pixmap), 750, 10);
		}else {
	  		pixmap = cui_pixmap_new_from_package_file("tux.png");
			cui_pixmap_set_xy(CUI_PIXMAP(pixmap), 750, 10);
		}
	}else{
	  	pixmap = cui_pixmap_new_from_package_file("tux.png");
		cui_pixmap_set_xy(CUI_PIXMAP(pixmap), 750, 10);
	}
	cui_widget_set_background(pixmap, t->background);
#endif
	return t;
}


void
terminal_apply_tag(Terminal *t, 
		gchar *tag,
		GtkTextMark *mbegin,
		GtkTextMark *mend)
{
	GtkTextIter begin, end;
	
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &begin, mbegin);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &end, mend);
	gtk_text_buffer_apply_tag_by_name(t->text_buffer, tag, &begin, &end);
}



void
terminal_set_working_directory(Terminal *t, GString *wd){
	GtkWidget *window; 
	gchar *title;
	
	/*
	 * We call this function every time the shell gives us a
	 * working directory.
	 */
	if (strcmp(t->wd->str, wd->str) == 0)
		return;
	
	t->wd = g_string_assign(t->wd, wd->str);
	window = lookup_widget(GTK_WIDGET(t->text_view), "app1");
	title = g_strdup_printf(_("CUI terminal (%s)"), wd->str);	
	gtk_window_set_title(GTK_WINDOW(window), title);
	
	dirhistory_directory_changed(wd);
	g_free(title);
}


void
terminal_set_prompt(Terminal *t,
		GString *prompt){
	gchar *tmp;
#ifdef DEBUG_TERM
	g_message("%s(): start with prompt: '%s'", __func__, prompt->str);
#endif
#if 0
FIXME:
	We have to carefully study the prompt on some distro and bash version, 
	then we can make a regular expression to slice it to parts. The most 
	important thing of course is the length of the prompt.
	
	/*
	 * The last part of the working directory as found in the
	 * prompt.
	 */
	tmp = g_utf8_strrchr(prompt->str, -1, '/');
	if (tmp == NULL){
		g_message("the / not found");
		tmp = g_utf8_strrchr(prompt->str, -1, ':');
	}
	if (tmp == NULL){
		g_message("the : not found");
		tmp = g_utf8_strrchr(prompt->str, -1, ']');
		if (tmp != NULL){
			g_message("found the ]");
			tmp = g_utf8_strrchr(prompt->str, tmp - prompt->str, ' ');
		}
	}			
	if (tmp != NULL)
		t->directory = g_string_assign(t->directory, tmp + 1);
	else
		g_warning("%s(): Directory name not found in the prompt.", __func__);

	/*
	 * The username in the prompt.
	 */
	t->username = g_string_assign(t->username, prompt->str);
	tmp = g_utf8_strchr(t->username->str, -1, '@');
	if (tmp != NULL){
		t->username = g_string_truncate(t->username, 
				g_utf8_strlen(t->username->str, -1) -
				g_utf8_strlen(tmp, -1));
	}
	/*
	 * The hostname.
	 */
	tmp = g_utf8_strrchr(prompt->str, -1, '@');
	if (tmp == 0)
		t->hostname = g_string_assign(t->hostname, prompt->str);
	else
		t->hostname = g_string_assign(t->hostname, tmp + 1);
	tmp = g_utf8_strchr(t->hostname->str, -1, ':');
	if (tmp != NULL){
		t->hostname = g_string_truncate(t->hostname, 
				g_utf8_strlen(t->hostname->str, -1) -
				g_utf8_strlen(tmp, -1));
	}
#endif	
	t->prompt_length = g_utf8_strlen(prompt->str, -1);
		/*
		g_utf8_strlen(t->username->str, -1) + 
		g_utf8_strlen(t->hostname->str, -1) + 
		g_utf8_strlen(t->directory->str, -1) +
		6;*/
#ifdef DEBUG_TERM
	g_message("%s(): username: '%s'", __func__, t->username->str);
	g_message("%s(): hostname: '%s'", __func__, t->hostname->str);
	g_message("%s(): directory: '%s'", __func__, t->directory->str);
	g_message("%s(): prompt_length=%d", __func__, t->prompt_length);
#endif
}

/** 
 * terminal_send_command:
 *
 * Emits a command to the terminal.
 */
gint 
terminal_send_command(Terminal *t, gchar *command, gboolean send)
{
	gsize written;
	GError *error = NULL;
	gchar *stored_command;
	
	g_return_val_if_fail(t != NULL, 0);
	g_return_val_if_fail(command != NULL, 0);
	/*
	 * If the shell is executing a command we store this one into a list.
	 */
	if (t->status != TermReady){
		stored_command = g_strdup(command);
		t->pending_commands = g_list_append(t->pending_commands,
                                             stored_command);
		return 0;
	}
	
	
	/* Sending End Ctrl+U to delete current line */
	g_io_channel_write_chars(t->shell_stream,
			"\033[4~\025", -1, &written, &error);
	/* Sendind the command */
	t->status = TermSendingCommand;
	g_io_channel_write_chars(t->shell_stream, 
			command, -1, &written, &error);
	if (send)
		g_io_channel_write_chars(t->shell_stream,
			"\n", -1, &written, &error);

	return written;
}


gint 
terminal_get_token_marks(Terminal *terminal,
		char *token,
		GtkTextMark **mbegin,
		GtkTextMark **mend)
{
	GtkTextIter begin, end;
	gchar *text;
	
	g_return_val_if_fail(terminal != NULL, 0);
	g_return_val_if_fail(terminal->lexer_cursor != NULL, 0);
		
	gtk_text_buffer_get_iter_at_mark(terminal->text_buffer, &begin, 
			terminal->lexer_cursor);
	gtk_text_buffer_get_iter_at_mark(terminal->text_buffer, &end, 
			terminal->lexer_cursor);
	gtk_text_iter_forward_chars(&end, g_utf8_strlen(token, -1));
	text = gtk_text_buffer_get_text(terminal->text_buffer, &begin, &end, 
			FALSE);

	if (strcmp(token, text) != 0)
		g_warning(_("%s: Searching token '%s', found '%s'\n"), 
				__func__, token, text);
	g_free(text);
	
	if (mbegin != NULL)
		*mbegin = terminal->lexer_cursor;
	terminal->lexer_cursor = gtk_text_buffer_create_mark(
			terminal->text_buffer, NULL, &end, FALSE);
	if (mend != NULL)
		*mend = terminal->lexer_cursor;

	return 0;
}


int terminal_get_char(Terminal *t, char *buf, int max_size){
	GtkTextIter end_iter;
	GtkTextIter iter, iter1;
	gchar *text;
	gint len;
	
	g_assert(t != NULL);
	gtk_text_buffer_get_end_iter(t->text_buffer, &end_iter);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, 
			t->parser_cursor);
	if (gtk_text_iter_compare(&iter, &end_iter) >= 0){
#ifdef DEBUG_TERM
		g_message("%s(): terminal->end mark reached", __func__);
#endif
		return 0;
	}
	
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter1,
					 t->parser_cursor);
	!gtk_text_iter_forward_char(&iter1);
	
	text = gtk_text_buffer_get_text(t->text_buffer, 
			&iter, &iter1, FALSE);
	gtk_text_buffer_move_mark(t->text_buffer, t->parser_cursor, &iter1);
	strcpy(buf, text);
	g_free(text);
	t->parsed_chars++;

	len = strlen(buf);
#ifdef DEBUG_TERM
	g_message("%s(): got string: '%s' %d", __func__, buf, buf[0]);
#endif
	return len;
}


gint 
terminal_command_starting(Terminal *t)
{
	GtkWidget *widget;
	GtkTextMark *cursor;
	GtkTextIter iter;
	GtkTextMark *mark;
#ifdef DEBUG_TERM
	g_message("%s(): Command is starting now.", __func__);
#endif
	/*
	 * If this was an empty command we can forget this whole thingy.
	 */
	if (g_utf8_strlen(t->command->str, -1) == 0)
		return 0;
	/*
	 * This variable shows if the terminal is currently running a command.
	 */ 
	t->status = TermExecuting;
	/*
	 * The abort button is sensitive when we running a command and can be
	 * used to stop this program (the same as the Ctrl+z keycode).
	 */
	widget = lookup_widget(GTK_WIDGET(t->text_view), "abort_toolbutton");
	gtk_widget_set_sensitive(widget, TRUE);
	/*
	 * Creating the CuiWindow for this command and setting its begin mark.
	 */
	cursor = gtk_text_buffer_get_insert(t->text_buffer);
	gtk_text_buffer_get_iter_at_mark(t->text_buffer, &iter, cursor);
	mark = gtk_text_buffer_create_mark(t->text_buffer, NULL, &iter, TRUE);
	t->last_command_window = (struct CuiWidget *)
		cui_window_new(t->background);
	cui_window_set_begin_mark(CUI_WINDOW(t->last_command_window), mark);
	return 0;
}

void 
terminal_send_init_commands(Terminal *t)
{
	static gunichar c;
	GString *buffer = g_string_new("");
	gchar *init_commands = bg_conf_get_string("/init/commands", "ls -lha");
	gchar *text = init_commands;

#ifdef DEBUG_TERM
	g_message("%s(): Start.", __func__);
	terminal_debug_print_status(t);	
#endif
	
	do {
		c = g_utf8_get_char(text);
		text = g_utf8_next_char(text);
		if (c != '\n' && c != '\0'){
			buffer = g_string_append_unichar(buffer, c);
		}else{
			if (buffer->len > 1)
				terminal_send_command(t, buffer->str, TRUE);
			buffer = g_string_assign(buffer, "");
		}
	} while (c != '\0');
	
	g_string_free(buffer, TRUE);
	g_free(init_commands);
}

gint 
terminal_command_ended(Terminal *t)
{
	GtkWidget *widget;
#ifdef DEBUG_TERM
	g_message("%s(): Command is completed now.", __func__);
#endif
	/*
	 * If this was an empty command we do nothing else. 
	 */
	if (g_utf8_strlen(t->command->str, -1) == 0){
#ifdef DEBUG
		g_warning("the command string is empty");
#endif
		return 0;
	}
	
	if(t->last_command_window == NULL){
		g_warning("%s(): There is no window to use", __func__);
		return -1;
	}
	
	/*
	 * If the command ended we can't use the abort toolbutton any more.
	 */
	widget = lookup_widget(GTK_WIDGET(t->text_view), "abort_toolbutton");
	gtk_widget_set_sensitive(widget, FALSE);
	
	terminal_redraw_now(t);
	
	cui_window_set_command(CUI_WINDOW(t->last_command_window), t->command);
	cui_window_set_working_directory(CUI_WINDOW(t->last_command_window), 
			t->wd);
	command_executed( (gpointer)t);

	return 0;
}


/***************************************************************************
 * Callback functions handling character terminal and shell I/O.           *
 *                                                                         *
 *                                                                         *
 **************************************************************************/
gboolean 
read_from_shell(GIOChannel * source, 
		GIOCondition condition, 
		gpointer data)
{
	Terminal *t = (Terminal *)(data);
	gchar current_line[10241];
	gsize could_read;
	GError *error = NULL;
#ifdef DEBUG_TERM
	g_message("%s() starting", __func__);
#endif
	/*
	 * If the stream is closed the shell must be exiting now.
	 */
	if (condition == G_IO_HUP)
		gtk_exit(EXIT_SUCCESS);
	/*
	 * Reading.
	 */
	g_io_channel_read_chars(source, current_line, 10240, &could_read, NULL);
	current_line[could_read] = 0;
	
#ifdef DEBUG_TERM
	g_message("%s(): could_read: %d", __func__, could_read);
	g_message("%s(): text: '%s'", __func__, current_line);
	gint n;
	for (n = 0; n < strlen(current_line); ++n)
		g_message("%s(): '%c' <%d>\n", __func__, 
				current_line[n],
				current_line[n]
				);
#endif
	/*
	 * Inserting to the text buffer.
	 */
	insert_to_terminal(t, current_line);
#ifdef DEBUG_TERM
	g_message("%s(): returning TRUE", __func__);
#endif
	return TRUE;
}

gboolean
on_terminal_key_release_event(GtkWidget * widget,
		GdkEventKey  *event,
		gpointer     user_data)
{
	/*
	 * Kedves Dávid Zoltán (kedz@gamma.ttk.pte.hu):
	 * Release special key(s) (SHIFT) in terminal structure.
	 */
	Terminal *terminal;
	
	terminal = (Terminal *)g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(terminal != NULL);
	
	if (event->type != GDK_KEY_RELEASE){
		g_warning("%s(): not a key release event\n", __func__);
		return FALSE;
	}

	switch (event->keyval){
		case GDK_Shift_L:
			terminal->hold_shift -= 3;
			//g_message("GDK_Shift_L release, hold_shift=%d", terminal->hold_shift);
			break;
		case GDK_Shift_R:
			terminal->hold_shift -= 1;
			//g_message("GDK_Shift_R release, hold_shift=%d", terminal->hold_shift);
			break;
	}
	return TRUE;
}

gboolean
on_terminal_key_press_event(GtkWidget  *widget, 
		GdkEventKey  *event, 
		gpointer     user_data)
{
	int retval;
	gsize written;
	GError *error = NULL;
	Terminal *terminal;

	terminal = (Terminal *)g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(terminal != NULL);
	
	if (event->type != GDK_KEY_PRESS){
		g_warning("%s(): not a key press event\n", __func__);
		return FALSE;
	}
#ifdef DEBUG_TERM_LOT
	g_message("----------------------------------------------------");
	g_message("event->keyval = %d", event->keyval);
	g_message("stringval[0]: %d", event->string[0]);
	g_message("stringval[1]: %d", event->string[1]);
	g_message("----------------------------------------------------");
#endif
	switch(event->keyval){
		/*
		 * Cursor arrows.
		 */
		case GDK_Up:
			g_io_channel_write_chars(terminal->shell_stream, 
					//"\033A", -1,
					"\033[A", -1,
                                 	&written, &error);
			break;
		case GDK_Down:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\033[B", -1,
					//"\033B", -1,
                                 	&written, &error);
			break;
		case GDK_Right:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\033[C", -1,
                                 	&written, &error);
			break;
		case GDK_Left:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\033[D", -1,
                                 	&written, &error);
			break;
		/*
		 * Other moving keys
		 */
		case GDK_Home:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\033[1~", -1,
                                 	&written, &error);
			break;
		case GDK_End:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\033[4~", -1,
                                 	&written, &error);
			break;
		case GDK_Page_Up:
			if (terminal->hold_shift == 0) 
				g_io_channel_write_chars(terminal->shell_stream, 
					"\033[5~", -1,
                                 	&written, &error);
			break;
		case GDK_Page_Down:
			if (terminal->hold_shift == 0)
				g_io_channel_write_chars(terminal->shell_stream, 
					"\033[6~", -1,
                                 	&written, &error);
			break;
		/*
		 * Erasers.
		 */
		case GDK_BackSpace:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\010", -1,
                                 	&written, &error);
			break;
		case GDK_Delete:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\033[3~", -1,
                                 	&written, &error);
			break;
		/*
		 * Other important keys.
		 */
		case GDK_Tab:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\t", -1,
                                 	&written, &error);
			break;
		case GDK_Escape:
			g_io_channel_write_chars(terminal->shell_stream, 
					"\033\001", -1,
                                 	&written, &error);
			break;
		case GDK_Return:
			terminal->status = TermSendingUserEnter;
			g_io_channel_write_chars(terminal->shell_stream, 
					event->string, -1,
                                 	&written, &error);
			break;
		/*
		 * Special keys. (SHIFT)
		 */
		case GDK_Shift_L:
			terminal->hold_shift += 3;
			//g_message("GDK_Shift_L press, hold_shift=%d", terminal->hold_shift);
			break;
		case GDK_Shift_R:
			terminal->hold_shift += 1;
			//g_message("GDK_Shift_R press, hold_shift=%d", terminal->hold_shift);
			break;
		default:
			g_io_channel_write_chars(terminal->shell_stream, 
					event->string, -1,
                                 	&written, &error);
	}

	if (terminal->hold_shift > 0) {
		GtkScrolledWindow *scrollw = (GtkScrolledWindow*) lookup_widget(
				GTK_WIDGET(widget), "scrolledwindow1");
		GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment(
				scrollw);
		gdouble val = vadj->value;
		if (event->keyval == GDK_Page_Up) {
			val -= vadj->page_size;
			if (val < vadj->lower)
				val = vadj->lower;
		}else if (event->keyval == GDK_Page_Down){
			val += vadj->page_size;
			if (val > (vadj->upper - vadj->page_size))
				val = vadj->upper - vadj->page_size;
		}
		gtk_adjustment_set_value(vadj, val);
	}

	if (error != NULL){
		fprintf(stderr, "%s(): Error writing to shell: '%s'\n",
				__func__,
				error->message);
		fflush(stderr);
	}
	
  	return TRUE;
}

/**
 * terminal_get_working_directory:
 * @terminal: a #Terminal
 * @returns: the name of the current working directory
 *
 * Returns the name of the current working directory from the BASH prompt. The
 * memory area returned should not be tampered.
 */
const gchar *
terminal_get_working_directory(Terminal *terminal)
{
	g_return_val_if_fail(terminal != NULL, NULL);
	return terminal->wd->str;
}

void
terminal_changed(Terminal *t)
{
#ifdef DEBUG_TERM
	terminal_debug_print_status(t);
#endif
	if (t->status == TermReady){
		type_timeout_function(t);
	}
	
	if (t->status == TermSendingCommand){
		type_timeout_function(t);
	}
}
