%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: w_lexer.l,v 1.1 2006/02/24 08:31:00 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: w_lexer.l,v $
 * Revision 1.1  2006/02/24 08:31:00  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "w_parser.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
int w_lexer_program = 0;

%}

%option yylineno

FS          [ \t]+
TIME        [0-9]{1,2}:[0-9][0-9]
UDATE       [0-9][0-9]:[0-9][0-9]:[0-9][0-9]
UTIME       [0-9]{1,2}:[0-9][0-9]
FROMUNKNOWN "-"
FROMDSPL    :[0-9]+\.[0-9]

%x PROGRAMSTATE

%%
	
	if (window_for_lex->parsed_chars == 0){
		cui_window_set_title(window_for_lex, _("Logged on Users"));
		BEGIN(0);
	}
	
	if (w_lexer_program != 0) {
		BEGIN(PROGRAMSTATE);
		w_lexer_program = 0;
	}

{FS}{UDATE}[^\n]+ {
	#ifdef DEBUG
	g_message("%s(): UPTIME: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		w_lval.sval.begin, 
		w_lval.sval.end); 
	return UPTIME;
}

USER{FS}TTY{FS}FROM{FS}LOGIN@{FS}IDLE{FS}[^\n]+ {
	#ifdef DEBUG
	g_message("%s(): HEADER: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		w_lval.sval.begin, 
		w_lval.sval.end); 
	return HEADER;
  }

^[a-zA-Z0-9]+ {
	#ifdef DEBUG
	g_message("%s(): NAME: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "username", 
		w_lval.sval.begin, 
		w_lval.sval.end); 
	return NAME;
}

(:[0-9]+)|([a-z]+\/[0-9]+) {
	#ifdef DEBUG
	g_message("%s(): LINE: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	return LINE;
}


{FROMUNKNOWN}|{FROMDSPL} {
	#ifdef DEBUG
	g_message("%s(): FROM: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	return FROM;
}


"?xdm?" {
	#ifdef DEBUG
	g_message("%s(): XDM: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	return XDM;
}

[0-9]+\.[0-9]+[smh] {
	#ifdef DEBUG
	g_message("%s(): INTERVAL: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	return INTERVAL;
}

{TIME}[smh]* {
	#ifdef DEBUG
	g_message("%s(): TIME: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	return TIME;
}

<PROGRAMSTATE>[^ ][^\n]* {
	#ifdef DEBUG
	g_message("%s(): PROGRAM: '%s'", __func__, yytext);
	#endif
	w_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		w_lval.sval.text,
		&w_lval.sval.begin,
		&w_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "command_name", 
		w_lval.sval.begin, 
		w_lval.sval.end);
	BEGIN(0);
	return PROGRAM;
}


<*>{FS} {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

\n+ {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

%%
