%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: who_lexer.l,v 1.1 2006/02/24 08:58:09 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: who_lexer.l,v $
 * Revision 1.1  2006/02/24 08:58:09  pipas
 * Initial revision
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "who_parser.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))

%}

%option yylineno

FS       [ \t]+
MONTH    [A-Z][a-z][a-z]
DATE     [0-9][0-9]*
FULLDATE [0-9]{4,4}-[0-9]{2,2}-[0-9]{2,2}
TIME     [0-9]{1,2}:[0-9][0-9]
%%
	cui_window_set_title(window_for_lex, _("Logged on Users"));

NAME{FS}LINE{FS}TIME{FS}COMMENT {
	#ifdef DEBUG
	g_message("%s(): HEADER: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		who_lval.sval.begin, 
		who_lval.sval.end); 
	return HEADER;
  }

^[a-zA-Z0-9]+ {
	#ifdef DEBUG
	g_message("%s(): NAME: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "username", 
		who_lval.sval.begin, 
		who_lval.sval.end); 
	return NAME;
}

(:[0-9]+)|([a-z]+\/[0-9]+) {
	#ifdef DEBUG
	g_message("%s(): LINE: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	return LINE;
}

{MONTH} {
	#ifdef DEBUG
	g_message("%s(): MONTH: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	return MONTH;
}

{DATE} {
	#ifdef DEBUG
	g_message("%s(): DATE: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	return DATE;
}

{FULLDATE} {
	#ifdef DEBUG
	g_message("%s(): FULLDATE: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	return FDATE;
}

{TIME} {
	#ifdef DEBUG
	g_message("%s(): TIME: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	return TIME;
}

\([^\(]+\) {
	#ifdef DEBUG
	g_message("%s(): COMMENT: '%s'", __func__, yytext);
	#endif
	who_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		who_lval.sval.text,
		&who_lval.sval.begin,
		&who_lval.sval.end);
	return COMMENT;
}

{FS} {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

\n+ {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

%%
